"""Test cases for :mod:`dispel.providers.ads.tasks.draw`."""

import pandas as pd
import pytest
from numpy.testing import assert_almost_equal

from dispel.data.flags import Flag
from dispel.providers.generic.tasks.draw.shapes import (
    extract_distance_axis_sc,
    extract_distance_axis_scc,
    get_max_dist_corner,
    get_reference_path,
    get_user_path,
    get_valid_up_sampled_path,
)
from dispel.providers.generic.tasks.draw.steps import (
    get_speed_accuracy,
    get_user_duration,
)
from tests.processing.helper import assert_level_values, assert_measure_value


@pytest.mark.parametrize(
    "level, flags",
    [
        ("square_counter_clock-right", []),
        ("square_counter_clock-right-02", []),
        ("square_clock-right", []),
        ("square_clock-right-02", []),
        ("infinity-right", []),
        ("infinity-right-02", []),
        ("spiral-right", []),
        ("spiral-right-02", []),
        ("square_counter_clock-left", []),
        ("square_counter_clock-left-02", []),
        ("square_clock-left", []),
        ("square_clock-left-02", []),
        ("infinity-left", []),
        ("infinity-left-02", []),
        ("spiral-left", []),
        ("spiral-left-02", []),
    ],
)
def test_draw_process_flags(example_reading_processed_draw, level, flags):
    """Ensure expected drawing does not have any flag."""
    invs = example_reading_processed_draw.get_level(level).get_flags()
    assert invs == flags


INVALID_FIXTURE = Flag(
    id_="draw-behavioral-deviation-upmo",
    reason="The phone has not been kept at a portrait upright mode "
    "({percentage}%) for more than 70.0% of the test at the level "
    "{level}.",
    stop_processing=False,
)


@pytest.mark.parametrize(
    "level, flags, percentage",
    [
        ("square_counter_clock-right", [INVALID_FIXTURE], 28.7),
        ("square_counter_clock-right-02", [INVALID_FIXTURE], 0.0),
        ("square_clock-right", [INVALID_FIXTURE], 65.3),
        ("square_clock-right-02", [INVALID_FIXTURE], 31.4),
        ("infinity-right", [INVALID_FIXTURE], 0.0),
        ("infinity-right-02", [INVALID_FIXTURE], 61.3),
        ("spiral-right", [], None),
        ("spiral-right-02", [INVALID_FIXTURE], 27.8),
        ("square_counter_clock-left", [], None),
        ("square_counter_clock-left-02", [INVALID_FIXTURE], 2.7),
        ("square_clock-left", [INVALID_FIXTURE], 0.0),
        ("square_clock-left-02", [INVALID_FIXTURE], 4.6),
        ("infinity-left", [], None),
        ("infinity-left-02", [], None),
        ("spiral-left", [], None),
        ("spiral-left-02", [INVALID_FIXTURE], 4.9),
    ],
)
def test_draw_process_invalid(
    example_reading_processed_draw_orientation, level, flags, percentage
):
    """Ensure the phone orientation is controlled with ads format."""
    invs = example_reading_processed_draw_orientation.get_level(level).get_flags()
    expected = [inv.format(level=level, percentage=percentage) for inv in flags]
    assert invs == expected


def test_not_continuous_draw_process(
    example_reading_processed_draw_not_continuous,
):
    """Ensure that non-continuous drawing are flagged."""
    invs = example_reading_processed_draw_not_continuous.get_level(
        "square_counter_clock-right"
    ).get_flags()
    assert len(invs) > 0
    assert invs[0].id == "draw-behavioral-deviation-continuous_drawing"


def test_get_user_duration(example_data_draw_sccr_screen):
    """Test the good computation of the user duration measure."""
    user_duration = get_user_duration(
        example_data_draw_sccr_screen.set_index("tsTouch")
    )
    assert user_duration == 3140.0


def test_get_speed_accuracy(paths_drawing_sccr_data, dtw_raw_data):
    """Test the good computation of the speed accuracy measure."""
    speed_accuracy = get_speed_accuracy(
        paths_drawing_sccr_data.set_index("tsTouch"),
        dtw_raw_data["dtw_data"]["dtw_mean_distance"],
    )
    assert speed_accuracy == pytest.approx(8.290898668348477e-05)


@pytest.mark.parametrize(
    "level,expected",
    [
        (
            "square_counter_clock-right",
            {
                "draw-right_scc_first-dur_acc-mean": 8.344722020302665e-05,
                "draw-right_scc_first-dur_acc-median": 9.505871417151748e-05,
                "draw-right_scc_first-sim-mean": 3.8164403416288613,
                "draw-right_scc_first-sim-median": 3.3502592619230027,
                "draw-right_scc_first-press-mean": 3.5327160358428955,
                "draw-right_scc_first-press-cv": 0.38867831230163574,
                "draw-right_scc_first-user_dur": 3140.0,
                "draw-right_scc_first-smooth": -2.8360552784979554,
                "draw-right_scc_first-rt": 424.0,
                "draw-right_scc_first-speed-mean": 496.60565353400926,
                "draw-right_scc_first-speed-std": 259.4948306522355,
                "draw-right_scc_first-speed-median": 499.3458766563266,
                "draw-right_scc_first-speed-min": 0.0,
                "draw-right_scc_first-speed-max": 1024.2278575897217,
                "draw-right_scc_first-speed-q95": 896.8024432659149,
                "draw-right_scc_first-speed-cv": 0.5225370045741221,
                "draw-right_scc_first-c_ratio": 0.9985204441890031,
                "draw-right_scc_first-itrem_ps_ene_diss": 0.0,
                "draw-right_scc_first-itrem_ps_ene_x_traj": 0.0,
                "draw-right_scc_first-itrem_ps_ene_y_traj": 0.0,
                "draw-right_scc_first-itrem_ps_peak_diss": 0.0,
                "draw-right_scc_first-itrem_ps_peak_x_traj": 0.0,
                "draw-right_scc_first-itrem_ps_peak_y_traj": 5.00020000800032,
                "draw-right_scc_first-itrem_ps_ent_diss": 1.3848212001926858,
                "draw-right_scc_first-itrem_ps_ent_x_traj": 1.3593880249979848,
                "draw-right_scc_first-itrem_ps_ent_y_traj": 0.8685618488278385,
                "draw-right_scc_first-itrem_ps_amp_diss": 0.05798658803252746,
                "draw-right_scc_first-itrem_ps_amp_x_traj": 0.057042676022729534,
                "draw-right_scc_first-itrem_ps_amp_y_traj": 194.9559801584133,
                "draw-right_scc_first-corner": 0.1705313896737278,
                "draw-right_scc_first-axes_over": 2.111575483460984,
                "draw-right_scc_first-raw_pacman_score": 0.896917148362235,
            },
        ),
        (
            "square_counter_clock-right-02",
            {
                "draw-right_scc_sec-dur_acc-mean": 4.274135010290627e-05,
                "draw-right_scc_sec-dur_acc-median": 5.0019016946781784e-05,
                "draw-right_scc_sec-sim-mean": 9.388662458792842,
                "draw-right_scc_sec-sim-median": 8.022630864101703,
                "draw-right_scc_sec-press-mean": 3.2097132205963135,
                "draw-right_scc_sec-press-cv": 0.3274546265602112,
                "draw-right_scc_sec-user_dur": 2492.0,
                "draw-right_scc_sec-smooth": -2.715783463570796,
                "draw-right_scc_sec-rt": 177.0,
                "draw-right_scc_sec-speed-mean": 627.0671735194019,
                "draw-right_scc_sec-speed-std": 383.1463370519542,
                "draw-right_scc_sec-speed-median": 601.7799297300708,
                "draw-right_scc_sec-speed-min": 0.0,
                "draw-right_scc_sec-speed-max": 1692.5019484299878,
                "draw-right_scc_sec-speed-q95": 1333.3327611287434,
                "draw-right_scc_sec-speed-cv": 0.6110132267035334,
                "draw-right_scc_sec-c_ratio": 1.0128753024819122,
                "draw-right_scc_sec-itrem_ps_ene_diss": 0.0,
                "draw-right_scc_sec-itrem_ps_ene_x_traj": 0.0,
                "draw-right_scc_sec-itrem_ps_ene_y_traj": 0.0,
                "draw-right_scc_sec-itrem_ps_peak_diss": 6.666933344000426,
                "draw-right_scc_sec-itrem_ps_peak_x_traj": 6.666933344000426,
                "draw-right_scc_sec-itrem_ps_peak_y_traj": 6.666933344000426,
                "draw-right_scc_sec-itrem_ps_ent_diss": 0.8382076222346488,
                "draw-right_scc_sec-itrem_ps_ent_x_traj": 0.8393943707677398,
                "draw-right_scc_sec-itrem_ps_ent_y_traj": 0.865266445469805,
                "draw-right_scc_sec-itrem_ps_amp_diss": 1.1211764096141226,
                "draw-right_scc_sec-itrem_ps_amp_x_traj": 1.1241168809617945,
                "draw-right_scc_sec-itrem_ps_amp_y_traj": 169.3890656972321,
                "draw-right_scc_sec-corner": 0.7423453547943802,
                "draw-right_scc_sec-axes_over": 6.69682454072565,
                "draw-right_scc_sec-raw_pacman_score": 0.41329479768786126,
            },
        ),
        (
            "square_clock-right",
            {
                "draw-right_sc_first-dur_acc-mean": 7.342733674747874e-05,
                "draw-right_sc_first-dur_acc-median": 9.628144459338775e-05,
                "draw-right_sc_first-sim-mean": 5.857593996101036,
                "draw-right_sc_first-sim-median": 4.467190211967994,
                "draw-right_sc_first-press-mean": 3.5338096618652344,
                "draw-right_sc_first-press-cv": 0.3021560609340668,
                "draw-right_sc_first-user_dur": 2325.0,
                "draw-right_sc_first-smooth": -1.6533999991381,
                "draw-right_sc_first-rt": 245.0,
                "draw-right_sc_first-speed-mean": 693.1880042309846,
                "draw-right_sc_first-speed-std": 498.4632807462466,
                "draw-right_sc_first-speed-median": 667.6947275797526,
                "draw-right_sc_first-speed-min": 0.0,
                "draw-right_sc_first-speed-max": 2154.4566521277793,
                "draw-right_sc_first-speed-q95": 1689.7005796432493,
                "draw-right_sc_first-speed-cv": 0.7190881517045818,
                "draw-right_sc_first-c_ratio": 1.0063437433332563,
                "draw-right_sc_first-itrem_ps_ene_diss": 0.0,
                "draw-right_sc_first-itrem_ps_ene_x_traj": 0.0,
                "draw-right_sc_first-itrem_ps_ene_y_traj": 0.0,
                "draw-right_sc_first-itrem_ps_peak_diss": 6.666933344000426,
                "draw-right_sc_first-itrem_ps_peak_x_traj": 6.666933344000426,
                "draw-right_sc_first-itrem_ps_peak_y_traj": 6.666933344000426,
                "draw-right_sc_first-itrem_ps_ent_diss": 0.44463322225020757,
                "draw-right_sc_first-itrem_ps_ent_x_traj": 0.44236044044817885,
                "draw-right_sc_first-itrem_ps_ent_y_traj": 0.7707299054331169,
                "draw-right_sc_first-itrem_ps_amp_diss": 5.010572497074841,
                "draw-right_sc_first-itrem_ps_amp_x_traj": 5.004303520381297,
                "draw-right_sc_first-itrem_ps_amp_y_traj": 198.0722283049864,
                "draw-right_sc_first-corner": -7.113944995820417,
                "draw-right_sc_first-axes_over": 1.1786706748726938,
                "draw-right_sc_first-raw_pacman_score": 0.815028901734104,
            },
        ),
        (
            "square_clock-right-02",
            {
                "draw-right_sc_sec-dur_acc-mean": 7.312864264367273e-05,
                "draw-right_sc_sec-dur_acc-median": 9.159452065815258e-05,
                "draw-right_sc_sec-sim-mean": 5.45454028440428,
                "draw-right_sc_sec-sim-median": 4.354879793873501,
                "draw-right_sc_sec-press-mean": 3.9900662899017334,
                "draw-right_sc_sec-press-cv": 0.32972246408462524,
                "draw-right_sc_sec-user_dur": 2507.0,
                "draw-right_sc_sec-smooth": -1.6434810527704868,
                "draw-right_sc_sec-rt": 96.0,
                "draw-right_sc_sec-speed-mean": 643.7671445328463,
                "draw-right_sc_sec-speed-std": 454.6905321463164,
                "draw-right_sc_sec-speed-median": 610.038104400136,
                "draw-right_sc_sec-speed-min": 0.0,
                "draw-right_sc_sec-speed-max": 1803.9227653952205,
                "draw-right_sc_sec-speed-q95": 1553.8799622479603,
                "draw-right_sc_sec-speed-cv": 0.7062965794507352,
                "draw-right_sc_sec-c_ratio": 1.023459066926929,
                "draw-right_sc_sec-itrem_ps_ene_diss": 0.0,
                "draw-right_sc_sec-itrem_ps_ene_x_traj": 0.0,
                "draw-right_sc_sec-itrem_ps_ene_y_traj": 0.0,
                "draw-right_sc_sec-itrem_ps_peak_diss": 0.0,
                "draw-right_sc_sec-itrem_ps_peak_x_traj": 0.0,
                "draw-right_sc_sec-itrem_ps_peak_y_traj": 6.666933344000426,
                "draw-right_sc_sec-itrem_ps_ent_diss": 1.2311973739700055,
                "draw-right_sc_sec-itrem_ps_ent_x_traj": 0.9182958340544896,
                "draw-right_sc_sec-itrem_ps_ent_y_traj": 0.8709545605149408,
                "draw-right_sc_sec-itrem_ps_amp_diss": 0.005330183816401224,
                "draw-right_sc_sec-itrem_ps_amp_x_traj": 0.005486801002219673,
                "draw-right_sc_sec-itrem_ps_amp_y_traj": 256.1125510697534,
                "draw-right_sc_sec-corner": 8.779079452596333,
                "draw-right_sc_sec-axes_over": 6.263632579904848,
                "draw-right_sc_sec-raw_pacman_score": 0.6859344894026975,
            },
        ),
        (
            "infinity-right",
            {
                "draw-right_inf_first-dur_acc-mean": 7.340502648773452e-05,
                "draw-right_inf_first-dur_acc-median": 8.807001840107322e-05,
                "draw-right_inf_first-sim-mean": 5.583215280798942,
                "draw-right_inf_first-sim-median": 4.653525376903706,
                "draw-right_inf_first-press-mean": 3.967913866043091,
                "draw-right_inf_first-press-cv": 0.32375451922416687,
                "draw-right_inf_first-user_dur": 2440.0,
                "draw-right_inf_first-smooth": -2.404486300129824,
                "draw-right_inf_first-rt": 330.0,
                "draw-right_inf_first-speed-mean": 599.3157889969134,
                "draw-right_inf_first-speed-std": 206.32391176907595,
                "draw-right_inf_first-speed-median": 589.4155169234556,
                "draw-right_inf_first-speed-min": 0.0,
                "draw-right_inf_first-speed-max": 1109.6575260162354,
                "draw-right_inf_first-speed-q95": 962.9330214332131,
                "draw-right_inf_first-speed-cv": 0.34426577032853467,
                "draw-right_inf_first-c_ratio": 0.990106035725694,
                "draw-right_inf_first-raw_pacman_score": 0.763,
            },
        ),
        (
            "infinity-right-02",
            {
                "draw-right_inf_sec-dur_acc-mean": 8.86364426973549e-05,
                "draw-right_inf_sec-dur_acc-median": 0.00012012068392575823,
                "draw-right_inf_sec-sim-mean": 4.941761374860795,
                "draw-right_inf_sec-sim-median": 3.6465006243019173,
                "draw-right_inf_sec-press-mean": 3.658088207244873,
                "draw-right_inf_sec-press-cv": 0.25026482343673706,
                "draw-right_inf_sec-user_dur": 2283.0,
                "draw-right_inf_sec-smooth": -2.3836730166973363,
                "draw-right_inf_sec-rt": 153.0,
                "draw-right_inf_sec-speed-mean": 659.5711126555981,
                "draw-right_inf_sec-speed-std": 175.8951466461087,
                "draw-right_inf_sec-speed-median": 658.8380477007697,
                "draw-right_inf_sec-speed-min": 0.0,
                "draw-right_inf_sec-speed-max": 1272.2430509679457,
                "draw-right_inf_sec-speed-q95": 913.0309504621168,
                "draw-right_inf_sec-speed-cv": 0.2666810951406148,
                "draw-right_inf_sec-c_ratio": 1.0033953207379653,
                "draw-right_inf_sec-raw_pacman_score": 0.744,
            },
        ),
        (
            "spiral-right",
            {
                "draw-right_spi_first-dur_acc-mean": 3.625731566849927e-05,
                "draw-right_spi_first-dur_acc-median": 4.578565580513369e-05,
                "draw-right_spi_first-sim-mean": 7.85325763235311,
                "draw-right_spi_first-sim-median": 6.218935515833581,
                "draw-right_spi_first-press-mean": 3.4971699714660645,
                "draw-right_spi_first-press-cv": 0.2855314612388611,
                "draw-right_spi_first-user_dur": 3512.0,
                "draw-right_spi_first-smooth": -2.196798369400713,
                "draw-right_spi_first-rt": 554.0,
                "draw-right_spi_first-speed-mean": 583.2960341203859,
                "draw-right_spi_first-speed-std": 183.75966329017663,
                "draw-right_spi_first-speed-median": 600.9250283241272,
                "draw-right_spi_first-speed-min": 0.0,
                "draw-right_spi_first-speed-max": 1147.3474502563477,
                "draw-right_spi_first-speed-q95": 882.4596405029297,
                "draw-right_spi_first-speed-cv": 0.3150367095625592,
                "draw-right_spi_first-c_ratio": 1.0366758036455654,
                "draw-right_spi_first-cross": 11.0,
                "draw-right_spi_first-cross_per_sec": 3.174603174603175,
                "draw-right_spi_first-cross_freq-mean": 4.595919485986745,
                "draw-right_spi_first-cross_freq-std": 2.723129525810282,
                "draw-right_spi_first-cross_freq-cv": 0.5925102765862804,
                "draw-right_spi_first-raw_pacman_score": 0.6880616174582799,
            },
        ),
        (
            "spiral-right-02",
            {
                "draw-right_spi_sec-dur_acc-mean": 4.4102456356181935e-05,
                "draw-right_spi_sec-dur_acc-median": 5.038156224448342e-05,
                "draw-right_spi_sec-sim-mean": 6.403409771351067,
                "draw-right_spi_sec-sim-median": 5.605346229665233,
                "draw-right_spi_sec-press-mean": 3.744626045227051,
                "draw-right_spi_sec-press-cv": 0.281756728887558,
                "draw-right_spi_sec-user_dur": 3541.0,
                "draw-right_spi_sec-smooth": -2.133374120057437,
                "draw-right_spi_sec-rt": 904.0,
                "draw-right_spi_sec-speed-mean": 550.757949379328,
                "draw-right_spi_sec-speed-std": 174.7847016348952,
                "draw-right_spi_sec-speed-median": 538.4124307071461,
                "draw-right_spi_sec-speed-min": 0.0,
                "draw-right_spi_sec-speed-max": 1400.0,
                "draw-right_spi_sec-speed-q95": 826.9935223234797,
                "draw-right_spi_sec-speed-cv": 0.31735302564741436,
                "draw-right_spi_sec-c_ratio": 0.9802616241484223,
                "draw-right_spi_sec-cross": 11.0,
                "draw-right_spi_sec-cross_per_sec": 3.141959440159954,
                "draw-right_spi_sec-cross_freq-mean": 4.051622872200597,
                "draw-right_spi_sec-cross_freq-std": 2.3324869049348402,
                "draw-right_spi_sec-cross_freq-cv": 0.5756920075011754,
                "draw-right_spi_sec-raw_pacman_score": 0.6803594351732991,
            },
        ),
        (
            "square_counter_clock-left",
            {
                "draw-left_scc_first-dur_acc-mean": 8.1262745431631e-05,
                "draw-left_scc_first-dur_acc-median": 9.674907160058269e-05,
                "draw-left_scc_first-sim-mean": 4.135000652409042,
                "draw-left_scc_first-sim-median": 3.4731238224545438,
                "draw-left_scc_first-press-mean": 3.263333559036255,
                "draw-left_scc_first-press-cv": 0.4033641219139099,
                "draw-left_scc_first-user_dur": 2976.0,
                "draw-left_scc_first-smooth": -2.5973813101558383,
                "draw-left_scc_first-rt": 759.0,
                "draw-left_scc_first-speed-mean": 524.9051479648346,
                "draw-left_scc_first-speed-std": 318.8244593985493,
                "draw-left_scc_first-speed-median": 500.3427929348415,
                "draw-left_scc_first-speed-min": 0.0,
                "draw-left_scc_first-speed-max": 1259.2637803819443,
                "draw-left_scc_first-speed-q95": 1068.6760980942668,
                "draw-left_scc_first-speed-cv": 0.6073944228489613,
                "draw-left_scc_first-c_ratio": 0.9896619451702262,
                "draw-left_scc_first-itrem_ps_ene_diss": 1.3582442890719373,
                "draw-left_scc_first-itrem_ps_ene_x_traj": 1.2059674507808522,
                "draw-left_scc_first-itrem_ps_ene_y_traj": 1351.3201982835178,
                "draw-left_scc_first-itrem_ps_peak_diss": 2.3077846190770708,
                "draw-left_scc_first-itrem_ps_peak_x_traj": 2.3077846190770708,
                "draw-left_scc_first-itrem_ps_peak_y_traj": 2.3077846190770708,
                "draw-left_scc_first-itrem_ps_ent_diss": 1.6415194378250797,
                "draw-left_scc_first-itrem_ps_ent_x_traj": 1.234776247262438,
                "draw-left_scc_first-itrem_ps_ent_y_traj": 1.017818400222274,
                "draw-left_scc_first-itrem_ps_amp_diss": 0.9757574347757905,
                "draw-left_scc_first-itrem_ps_amp_x_traj": 1.019434614815008,
                "draw-left_scc_first-itrem_ps_amp_y_traj": 985.0269017003152,
                "draw-left_scc_first-corner": 2.5197398747505324,
                "draw-left_scc_first-axes_over": 3.0425105096510237,
                "draw-left_scc_first-raw_pacman_score": 0.8901734104046243,
            },
        ),
        (
            "square_counter_clock-left-02",
            {
                "draw-left_scc_sec-dur_acc-mean": 5.301375835308396e-05,
                "draw-left_scc_sec-dur_acc-median": 5.0919519017132215e-05,
                "draw-left_scc_sec-sim-mean": 7.67413663357747,
                "draw-left_scc_sec-sim-median": 7.989761743903019,
                "draw-left_scc_sec-press-mean": 3.649440288543701,
                "draw-left_scc_sec-press-cv": 0.37275364995002747,
                "draw-left_scc_sec-user_dur": 2458.0,
                "draw-left_scc_sec-smooth": -2.698834912293684,
                "draw-left_scc_sec-rt": 261.0,
                "draw-left_scc_sec-speed-mean": 625.8069633857941,
                "draw-left_scc_sec-speed-std": 434.3044481514985,
                "draw-left_scc_sec-speed-median": 583.3339691162109,
                "draw-left_scc_sec-speed-min": 0.0,
                "draw-left_scc_sec-speed-max": 1628.514906939338,
                "draw-left_scc_sec-speed-q95": 1442.386952638626,
                "draw-left_scc_sec-speed-cv": 0.693991076420415,
                "draw-left_scc_sec-c_ratio": 0.9893625944213778,
                "draw-left_scc_sec-itrem_ps_ene_diss": 0.023595151925649992,
                "draw-left_scc_sec-itrem_ps_ene_x_traj": 0.10775965776569368,
                "draw-left_scc_sec-itrem_ps_ene_y_traj": 1305.7233583118496,
                "draw-left_scc_sec-itrem_ps_peak_diss": 3.529552946823755,
                "draw-left_scc_sec-itrem_ps_peak_x_traj": 0.0,
                "draw-left_scc_sec-itrem_ps_peak_y_traj": 3.529552946823755,
                "draw-left_scc_sec-itrem_ps_ent_diss": 2.3977985458136977,
                "draw-left_scc_sec-itrem_ps_ent_x_traj": 1.4198169075614602,
                "draw-left_scc_sec-itrem_ps_ent_y_traj": 0.7161076278013755,
                "draw-left_scc_sec-itrem_ps_amp_diss": 0.013370051267758015,
                "draw-left_scc_sec-itrem_ps_amp_x_traj": 0.09025700014124971,
                "draw-left_scc_sec-itrem_ps_amp_y_traj": 739.8803066472597,
                "draw-left_scc_sec-corner": 0.4095934138672363,
                "draw-left_scc_sec-axes_over": 5.395937315097465,
                "draw-left_scc_sec-raw_pacman_score": 0.5144508670520231,
            },
        ),
        (
            "square_clock-left",
            {
                "draw-left_sc_first-dur_acc-mean": 5.8814285155052026e-05,
                "draw-left_sc_first-dur_acc-median": 6.771726663517626e-05,
                "draw-left_sc_first-sim-mean": 6.911655288659621,
                "draw-left_sc_first-sim-median": 6.002960326657495,
                "draw-left_sc_first-press-mean": 3.766218900680542,
                "draw-left_sc_first-press-cv": 0.3523399233818054,
                "draw-left_sc_first-user_dur": 2460.0,
                "draw-left_sc_first-smooth": -1.6466649701280331,
                "draw-left_sc_first-rt": 378.0,
                "draw-left_sc_first-speed-mean": 618.929901627831,
                "draw-left_sc_first-speed-std": 474.3041904351159,
                "draw-left_sc_first-speed-median": 532.2969791935939,
                "draw-left_sc_first-speed-min": 0.0,
                "draw-left_sc_first-speed-max": 1936.5089053199404,
                "draw-left_sc_first-speed-q95": 1500.000991821289,
                "draw-left_sc_first-speed-cv": 0.7663294166070522,
                "draw-left_sc_first-c_ratio": 0.980672294619934,
                "draw-left_sc_first-itrem_ps_ene_diss": 0.0,
                "draw-left_sc_first-itrem_ps_ene_x_traj": 0.0,
                "draw-left_sc_first-itrem_ps_ene_y_traj": 0.0,
                "draw-left_sc_first-itrem_ps_peak_diss": 5.454763645091258,
                "draw-left_sc_first-itrem_ps_peak_x_traj": 5.454763645091258,
                "draw-left_sc_first-itrem_ps_peak_y_traj": 5.454763645091258,
                "draw-left_sc_first-itrem_ps_ent_diss": 0.7859562235372111,
                "draw-left_sc_first-itrem_ps_ent_x_traj": 0.8243881435364508,
                "draw-left_sc_first-itrem_ps_ent_y_traj": 0.9524353692875628,
                "draw-left_sc_first-itrem_ps_amp_diss": 1.0322536790066992,
                "draw-left_sc_first-itrem_ps_amp_x_traj": 1.09013967059756,
                "draw-left_sc_first-itrem_ps_amp_y_traj": 282.2062024458761,
                "draw-left_sc_first-corner": -0.8082457260651195,
                "draw-left_sc_first-axes_over": 6.5220309512468715,
                "draw-left_sc_first-raw_pacman_score": 0.5963391136801541,
            },
        ),
        (
            "square_clock-left-02",
            {
                "draw-left_sc_sec-dur_acc-mean": 7.808231438943117e-05,
                "draw-left_sc_sec-dur_acc-median": 9.497579405716134e-05,
                "draw-left_sc_sec-sim-mean": 5.595018538079564,
                "draw-left_sc_sec-sim-median": 4.599824627337066,
                "draw-left_sc_sec-press-mean": 3.479951858520508,
                "draw-left_sc_sec-press-cv": 0.3565198481082916,
                "draw-left_sc_sec-user_dur": 2289.0,
                "draw-left_sc_sec-smooth": -1.6558413709899116,
                "draw-left_sc_sec-rt": 111.0,
                "draw-left_sc_sec-speed-mean": 669.9442922031626,
                "draw-left_sc_sec-speed-std": 539.5158995925339,
                "draw-left_sc_sec-speed-median": 627.7584749109604,
                "draw-left_sc_sec-speed-min": 0.0,
                "draw-left_sc_sec-speed-max": 1958.3358764648438,
                "draw-left_sc_sec-speed-q95": 1741.8090471255232,
                "draw-left_sc_sec-speed-cv": 0.8053145699895359,
                "draw-left_sc_sec-c_ratio": 0.9867814787417858,
                "draw-left_sc_sec-itrem_ps_ene_diss": 18.61554097219272,
                "draw-left_sc_sec-itrem_ps_ene_x_traj": 18.793206951215936,
                "draw-left_sc_sec-itrem_ps_ene_y_traj": 323.6250708655474,
                "draw-left_sc_sec-itrem_ps_peak_diss": 4.285885721143131,
                "draw-left_sc_sec-itrem_ps_peak_x_traj": 4.285885721143131,
                "draw-left_sc_sec-itrem_ps_peak_y_traj": 4.285885721143131,
                "draw-left_sc_sec-itrem_ps_ent_diss": 0.8687241960806721,
                "draw-left_sc_sec-itrem_ps_ent_x_traj": 0.8463323942652123,
                "draw-left_sc_sec-itrem_ps_ent_y_traj": 1.0069463098206637,
                "draw-left_sc_sec-itrem_ps_amp_diss": 8.686904963591788,
                "draw-left_sc_sec-itrem_ps_amp_x_traj": 8.769812437371014,
                "draw-left_sc_sec-itrem_ps_amp_y_traj": 151.01899206926598,
                "draw-left_sc_sec-corner": -12.057977849542686,
                "draw-left_sc_sec-axes_over": -1.8551085817788362,
                "draw-left_sc_sec-raw_pacman_score": 0.6416184971098265,
            },
        ),
        (
            "infinity-left",
            {
                "draw-left_inf_first-dur_acc-mean": 4.5546207100114775e-05,
                "draw-left_inf_first-dur_acc-median": 0.00010090956862666208,
                "draw-left_inf_first-sim-mean": 8.539760811542822,
                "draw-left_inf_first-sim-median": 3.8544780222676045,
                "draw-left_inf_first-press-mean": 3.613333225250244,
                "draw-left_inf_first-press-cv": 0.4502080976963043,
                "draw-left_inf_first-user_dur": 2571.0,
                "draw-left_inf_first-smooth": -2.3932714219511984,
                "draw-left_inf_first-rt": 330.0,
                "draw-left_inf_first-speed-mean": 564.3864920100788,
                "draw-left_inf_first-speed-std": 148.24557520255806,
                "draw-left_inf_first-speed-median": 571.8299018012152,
                "draw-left_inf_first-speed-min": 0.0,
                "draw-left_inf_first-speed-max": 955.1017761230469,
                "draw-left_inf_first-speed-q95": 772.9972426096597,
                "draw-left_inf_first-speed-cv": 0.26266676701381914,
                "draw-left_inf_first-c_ratio": 0.9904257466667483,
                "draw-left_inf_first-raw_pacman_score": 0.695,
            },
        ),
        (
            "infinity-left-02",
            {
                "draw-left_inf_sec-dur_acc-mean": 4.319991162956622e-05,
                "draw-left_inf_sec-dur_acc-median": 4.878286919307572e-05,
                "draw-left_inf_sec-sim-mean": 8.469884925127028,
                "draw-left_inf_sec-sim-median": 7.50054858048445,
                "draw-left_inf_sec-press-mean": 3.2475461959838867,
                "draw-left_inf_sec-press-cv": 0.33963868021965027,
                "draw-left_inf_sec-user_dur": 2733.0,
                "draw-left_inf_sec-smooth": -2.4130316004526997,
                "draw-left_inf_sec-rt": 219.0,
                "draw-left_inf_sec-speed-mean": 549.4000573155911,
                "draw-left_inf_sec-speed-std": 156.6283387814699,
                "draw-left_inf_sec-speed-median": 540.0405491099638,
                "draw-left_inf_sec-speed-min": 125.0,
                "draw-left_inf_sec-speed-max": 979.1717529296875,
                "draw-left_inf_sec-speed-q95": 802.3021219290938,
                "draw-left_inf_sec-speed-cv": 0.28508977510262273,
                "draw-left_inf_sec-c_ratio": 1.0134366635626593,
                "draw-left_inf_sec-raw_pacman_score": 0.577,
            },
        ),
        (
            "spiral-left",
            {
                "draw-left_spi_first-dur_acc-mean": 3.960221187961974e-05,
                "draw-left_spi_first-dur_acc-median": 4.529553557726856e-05,
                "draw-left_spi_first-sim-mean": 5.623856311296229,
                "draw-left_spi_first-sim-median": 4.916977940145164,
                "draw-left_spi_first-press-mean": 2.5547356605529785,
                "draw-left_spi_first-press-cv": 0.4088081419467926,
                "draw-left_spi_first-user_dur": 4490.0,
                "draw-left_spi_first-smooth": -2.1337685654597505,
                "draw-left_spi_first-rt": 347.0,
                "draw-left_spi_first-speed-mean": 450.3776261948849,
                "draw-left_spi_first-speed-std": 144.63880364035577,
                "draw-left_spi_first-speed-median": 449.84800815582275,
                "draw-left_spi_first-speed-min": 0.0,
                "draw-left_spi_first-speed-max": 853.5041075486403,
                "draw-left_spi_first-speed-q95": 693.7034350984234,
                "draw-left_spi_first-speed-cv": 0.3211500643634737,
                "draw-left_spi_first-c_ratio": 1.0186800347492018,
                "draw-left_spi_first-cross": 11.0,
                "draw-left_spi_first-cross_per_sec": 2.4813895781637716,
                "draw-left_spi_first-cross_freq-mean": 9.44198428085613,
                "draw-left_spi_first-cross_freq-std": 11.977906290788649,
                "draw-left_spi_first-cross_freq-cv": 1.2685793509605994,
                "draw-left_spi_first-raw_pacman_score": 0.7804878048780488,
            },
        ),
        (
            "spiral-left-02",
            {
                "draw-left_spi_sec-dur_acc-mean": 2.748566291652509e-05,
                "draw-left_spi_sec-dur_acc-median": 3.358807495122353e-05,
                "draw-left_spi_sec-sim-mean": 8.289497466277457,
                "draw-left_spi_sec-sim-median": 6.783429340215619,
                "draw-left_spi_sec-press-mean": 2.5939624309539795,
                "draw-left_spi_sec-press-cv": 0.3632742166519165,
                "draw-left_spi_sec-user_dur": 4389.0,
                "draw-left_spi_sec-smooth": -2.0627821922132057,
                "draw-left_spi_sec-rt": 162.0,
                "draw-left_spi_sec-speed-mean": 457.41036477791687,
                "draw-left_spi_sec-speed-std": 122.71927943683062,
                "draw-left_spi_sec-speed-median": 452.6830561020795,
                "draw-left_spi_sec-speed-min": 0.0,
                "draw-left_spi_sec-speed-max": 929.7553062438965,
                "draw-left_spi_sec-speed-q95": 666.046446164449,
                "draw-left_spi_sec-speed-cv": 0.26829142688188456,
                "draw-left_spi_sec-c_ratio": 0.9989556182720133,
                "draw-left_spi_sec-cross": 8,
                "draw-left_spi_sec-cross_per_sec": 1.8331805682859763,
                "draw-left_spi_sec-cross_freq-mean": 5.821238963278483,
                "draw-left_spi_sec-cross_freq-std": 10.427057602828814,
                "draw-left_spi_sec-cross_freq-cv": 1.791209340246078,
                "draw-left_spi_sec-raw_pacman_score": 0.5686777920410783,
            },
        ),
        (
            "left-all",
            {
                "draw-left-acc_sig_ene": 0.99609596,
                "draw-left-acc_ps_ene_x": 0.00036317024491694383,
                "draw-left-acc_ps_ene_y": 0.0001748949219826379,
                "draw-left-acc_ps_ene_z": 0.0006941195207055273,
                "draw-left-acc_ps_ene_xyz": 0.0006171553090350557,
                "draw-left-acc_ps_peak_x": 2.65625,
                "draw-left-acc_ps_peak_y": 2.734375,
                "draw-left-acc_ps_peak_z": 9.84375,
                "draw-left-acc_ps_peak_xyz": 1.015625,
                "draw-left-acc_ps_ent_x": 6.707259246100832,
                "draw-left-acc_ps_ent_y": 6.621315953411637,
                "draw-left-acc_ps_ent_z": 6.604765211808896,
                "draw-left-acc_ps_ent_xyz": 6.700240236943751,
                "draw-left-acc_ps_amp_x": 2.030562791333068e-05,
                "draw-left-acc_ps_amp_y": 1.1974926565017086e-05,
                "draw-left-acc_ps_amp_z": 6.993526767473668e-05,
                "draw-left-acc_ps_amp_xyz": 3.0564679036615416e-05,
                "draw-left-gyr_sig_ene": 3.9556339,
                "draw-left-gyr_ps_ene_x": 0.005350254394898002,
                "draw-left-gyr_ps_ene_y": 0.004607988057578893,
                "draw-left-gyr_ps_ene_z": 0.0021451126092308925,
                "draw-left-gyr_ps_ene_xyz": 0.004954453435956907,
                "draw-left-gyr_ps_peak_x": 0.625,
                "draw-left-gyr_ps_peak_y": 0.625,
                "draw-left-gyr_ps_peak_z": 0.3125,
                "draw-left-gyr_ps_peak_xyz": 0.3125,
                "draw-left-gyr_ps_ent_x": 6.378163226491344,
                "draw-left-gyr_ps_ent_y": 6.249042510270051,
                "draw-left-gyr_ps_ent_z": 5.537502820989228,
                "draw-left-gyr_ps_ent_xyz": 6.446902487488826,
                "draw-left-gyr_ps_amp_x": 0.0009336828370578587,
                "draw-left-gyr_ps_amp_y": 0.0010181633988395333,
                "draw-left-gyr_ps_amp_z": 0.0009285384439863265,
                "draw-left-gyr_ps_amp_xyz": 0.0007242027204483747,
            },
        ),
        (
            "right-all",
            {
                "draw-right-acc_sig_ene": 1.1468403,
                "draw-right-acc_ps_ene_x": 0.0004213690067489395,
                "draw-right-acc_ps_ene_y": 0.00034760895372354383,
                "draw-right-acc_ps_ene_z": 0.0014522721162713026,
                "draw-right-acc_ps_ene_xyz": 0.0009450604953809005,
                "draw-right-acc_ps_peak_x": 0.3125,
                "draw-right-acc_ps_peak_y": 0.3125,
                "draw-right-acc_ps_peak_z": 9.140625,
                "draw-right-acc_ps_peak_xyz": 0.3125,
                "draw-right-acc_ps_ent_x": 6.501111148974417,
                "draw-right-acc_ps_ent_y": 6.350724561360384,
                "draw-right-acc_ps_ent_z": 6.796046465510675,
                "draw-right-acc_ps_ent_xyz": 6.623408425572533,
                "draw-right-acc_ps_amp_x": 5.972565486445092e-05,
                "draw-right-acc_ps_amp_y": 7.907166582299396e-05,
                "draw-right-acc_ps_amp_z": 8.151026122504845e-05,
                "draw-right-acc_ps_amp_xyz": 0.00011475952487671748,
                "draw-right-gyr_sig_ene": 5.2672396,
                "draw-right-gyr_ps_ene_x": 0.010967037359392862,
                "draw-right-gyr_ps_ene_y": 0.00888796242485057,
                "draw-right-gyr_ps_ene_z": 0.0025238500355939664,
                "draw-right-gyr_ps_ene_xyz": 0.008560635531722482,
                "draw-right-gyr_ps_peak_x": 0.3125,
                "draw-right-gyr_ps_peak_y": 0.234375,
                "draw-right-gyr_ps_peak_z": 0.3125,
                "draw-right-gyr_ps_peak_xyz": 0.3125,
                "draw-right-gyr_ps_ent_x": 5.394929780645835,
                "draw-right-gyr_ps_ent_y": 5.825805463163217,
                "draw-right-gyr_ps_ent_z": 4.554175449688035,
                "draw-right-gyr_ps_ent_xyz": 6.121190567638136,
                "draw-right-gyr_ps_amp_x": 0.008725795894861221,
                "draw-right-gyr_ps_amp_y": 0.0030497508123517036,
                "draw-right-gyr_ps_amp_z": 0.004580325912684202,
                "draw-right-gyr_ps_amp_xyz": 0.0016105580143630505,
            },
        ),
        (
            "all_levels",
            {
                "draw-press-mean": 3.3860617,
                "draw-press-std": 1.2637655,
                "draw-press-median": 3.4666667,
                "draw-press-min": 0.0,
                "draw-press-max": 6.1,
                "draw-press-cv": 0.37322578,
                "draw-rt-mean": 340.625,
                "draw-rt-std": 228.51721306428246,
                "draw-rt-median": 295.5,
                "draw-rt-min": 96.0,
                "draw-rt-max": 904.0,
            },
        ),
    ],
)
def test_draw_process(example_reading_processed_draw, level, expected):
    """Unit test to ensure the drawing measures are well computed."""
    assert_level_values(example_reading_processed_draw, level, expected)


@pytest.mark.parametrize(
    "level,expected",
    [
        (
            "square_counter_clock-right",
            {
                "draw-right_scc_first-dur_acc-mean": 7.311260236920608e-05,
                "draw-right_scc_first-dur_acc-median": 7.504668544923673e-05,
                "draw-right_scc_first-sim-mean": 6.811520202941694,
                "draw-right_scc_first-sim-median": 6.635975528384313,
                "draw-right_scc_first-press-mean": 1.7517077922821045,
                "draw-right_scc_first-press-cv": 0.45656588673591614,
                "draw-right_scc_first-user_dur": 2008.0,
                "draw-right_scc_first-smooth": -2.927633371296985,
                "draw-right_scc_first-rt": 528.0,
                "draw-right_scc_first-speed-mean": 744.744123524597,
                "draw-right_scc_first-speed-std": 547.4853787448133,
                "draw-right_scc_first-speed-median": 625.3471970558167,
                "draw-right_scc_first-speed-min": 0.0,
                "draw-right_scc_first-speed-max": 2708.415985107422,
                "draw-right_scc_first-speed-q95": 2176.470588235294,
                "draw-right_scc_first-speed-cv": 0.7351321903068783,
                "draw-right_scc_first-c_ratio": 0.9597239979281451,
                "draw-right_scc_first-itrem_ps_ene_diss": 0.0,
                "draw-right_scc_first-itrem_ps_ene_x_traj": 0.0,
                "draw-right_scc_first-itrem_ps_ene_y_traj": 0.0,
                "draw-right_scc_first-itrem_ps_peak_diss": 5.00020000800032,
                "draw-right_scc_first-itrem_ps_peak_x_traj": 5.00020000800032,
                "draw-right_scc_first-itrem_ps_peak_y_traj": 5.00020000800032,
                "draw-right_scc_first-itrem_ps_ent_diss": 1.407041074618404,
                "draw-right_scc_first-itrem_ps_ent_x_traj": 1.2861196240435047,
                "draw-right_scc_first-itrem_ps_ent_y_traj": 1.0081663573246264,
                "draw-right_scc_first-itrem_ps_amp_diss": 2.989640194750059,
                "draw-right_scc_first-itrem_ps_amp_x_traj": 3.39394217621226,
                "draw-right_scc_first-itrem_ps_amp_y_traj": 145.49419742400784,
                "draw-right_scc_first-corner": -19.17018179416824,
                "draw-right_scc_first-axes_over": 1.1338481616596852,
                "draw-right_scc_first-raw_pacman_score": 0.7090558766859345,
            },
        ),
        (
            "square_counter_clock-right-02",
            {
                "draw-right_scc_sec-dur_acc-mean": 0.0001222418216057742,
                "draw-right_scc_sec-dur_acc-median": 0.0001314122047484273,
                "draw-right_scc_sec-sim-mean": 3.878855555869872,
                "draw-right_scc_sec-sim-median": 3.6081760427270027,
                "draw-right_scc_sec-press-mean": 1.8898111581802368,
                "draw-right_scc_sec-press-cv": 0.39239397644996643,
                "draw-right_scc_sec-user_dur": 2109.0,
                "draw-right_scc_sec-smooth": -2.635083192693691,
                "draw-right_scc_sec-rt": 127.0,
                "draw-right_scc_sec-speed-mean": 734.1989503594647,
                "draw-right_scc_sec-speed-std": 639.0800549741042,
                "draw-right_scc_sec-speed-median": 612.8823336432962,
                "draw-right_scc_sec-speed-min": 0.0,
                "draw-right_scc_sec-speed-max": 3003.54266166687,
                "draw-right_scc_sec-speed-q95": 2435.6622583725857,
                "draw-right_scc_sec-speed-cv": 0.8704453400011126,
                "draw-right_scc_sec-c_ratio": 0.9967932801487398,
                "draw-right_scc_sec-itrem_ps_ene_diss": 0.0,
                "draw-right_scc_sec-itrem_ps_ene_x_traj": 0.0,
                "draw-right_scc_sec-itrem_ps_ene_y_traj": 0.0,
                "draw-right_scc_sec-itrem_ps_peak_diss": 6.000240009600384,
                "draw-right_scc_sec-itrem_ps_peak_x_traj": 6.000240009600384,
                "draw-right_scc_sec-itrem_ps_peak_y_traj": 6.000240009600384,
                "draw-right_scc_sec-itrem_ps_ent_diss": 1.0903301021665768,
                "draw-right_scc_sec-itrem_ps_ent_x_traj": 1.0864200257257706,
                "draw-right_scc_sec-itrem_ps_ent_y_traj": 0.9665466563101308,
                "draw-right_scc_sec-itrem_ps_amp_diss": 1.023666888538086,
                "draw-right_scc_sec-itrem_ps_amp_x_traj": 1.0338629996494095,
                "draw-right_scc_sec-itrem_ps_amp_y_traj": 220.2469032516891,
                "draw-right_scc_sec-corner": -5.6759720459795355,
                "draw-right_scc_sec-axes_over": 4.928068455573616,
                "draw-right_scc_sec-raw_pacman_score": 0.8863198458574181,
            },
        ),
        (
            "square_clock-right",
            {
                "draw-right_sc_first-dur_acc-mean": 0.00011621383290339719,
                "draw-right_sc_first-dur_acc-median": 0.00010677631845446981,
                "draw-right_sc_first-sim-mean": 4.0190694194031895,
                "draw-right_sc_first-sim-median": 4.37429824041778,
                "draw-right_sc_first-press-mean": 1.8251601457595825,
                "draw-right_sc_first-press-cv": 0.4356513023376465,
                "draw-right_sc_first-user_dur": 2141.0,
                "draw-right_sc_first-smooth": -1.66205898038348,
                "draw-right_sc_first-rt": 262.0,
                "draw-right_sc_first-speed-mean": 723.6063713502324,
                "draw-right_sc_first-speed-std": 598.4136872256568,
                "draw-right_sc_first-speed-median": 589.5408742568072,
                "draw-right_sc_first-speed-min": 0.0,
                "draw-right_sc_first-speed-max": 2628.622391644646,
                "draw-right_sc_first-speed-q95": 2195.3260085161987,
                "draw-right_sc_first-speed-cv": 0.8269878637318118,
                "draw-right_sc_first-c_ratio": 0.9970320680094824,
                "draw-right_sc_first-itrem_ps_ene_diss": 0.0,
                "draw-right_sc_first-itrem_ps_ene_x_traj": 0.0,
                "draw-right_sc_first-itrem_ps_ene_y_traj": 0.0,
                "draw-right_sc_first-itrem_ps_peak_diss": 6.666933344000426,
                "draw-right_sc_first-itrem_ps_peak_x_traj": 6.666933344000426,
                "draw-right_sc_first-itrem_ps_peak_y_traj": 6.666933344000426,
                "draw-right_sc_first-itrem_ps_ent_diss": 1.2279492599164927,
                "draw-right_sc_first-itrem_ps_ent_x_traj": 1.2129962998220178,
                "draw-right_sc_first-itrem_ps_ent_y_traj": 0.7940311975589955,
                "draw-right_sc_first-itrem_ps_amp_diss": 0.551144129197221,
                "draw-right_sc_first-itrem_ps_amp_x_traj": 0.5538884621470451,
                "draw-right_sc_first-itrem_ps_amp_y_traj": 195.205952506232,
                "draw-right_sc_first-corner": -2.0782852117241033,
                "draw-right_sc_first-axes_over": 4.4458739395850655,
                "draw-right_sc_first-raw_pacman_score": 0.9720616570327553,
            },
        ),
        (
            "square_clock-right-02",
            {
                "draw-right_sc_sec-dur_acc-mean": 6.389462505278238e-05,
                "draw-right_sc_sec-dur_acc-median": 5.668410550776475e-05,
                "draw-right_sc_sec-sim-mean": 7.668186542512734,
                "draw-right_sc_sec-sim-median": 8.643620633680587,
                "draw-right_sc_sec-press-mean": 1.990423560142517,
                "draw-right_sc_sec-press-cv": 0.4122406244277954,
                "draw-right_sc_sec-user_dur": 2041.0,
                "draw-right_sc_sec-smooth": -1.6595499785880885,
                "draw-right_sc_sec-rt": 128.0,
                "draw-right_sc_sec-speed-mean": 760.1806681612617,
                "draw-right_sc_sec-speed-std": 622.4770440328801,
                "draw-right_sc_sec-speed-median": 645.8330154418945,
                "draw-right_sc_sec-speed-min": 0.0,
                "draw-right_sc_sec-speed-max": 2645.832061767578,
                "draw-right_sc_sec-speed-q95": 2366.617180319393,
                "draw-right_sc_sec-speed-cv": 0.8188540831201858,
                "draw-right_sc_sec-c_ratio": 0.9978237788653004,
                "draw-right_sc_sec-itrem_ps_ene_diss": 0.0,
                "draw-right_sc_sec-itrem_ps_ene_x_traj": 0.0,
                "draw-right_sc_sec-itrem_ps_ene_y_traj": 0.0,
                "draw-right_sc_sec-itrem_ps_peak_diss": 6.666933344000426,
                "draw-right_sc_sec-itrem_ps_peak_x_traj": 6.666933344000426,
                "draw-right_sc_sec-itrem_ps_peak_y_traj": 6.666933344000426,
                "draw-right_sc_sec-itrem_ps_ent_diss": 1.3588701214805292,
                "draw-right_sc_sec-itrem_ps_ent_x_traj": 1.3623049247986576,
                "draw-right_sc_sec-itrem_ps_ent_y_traj": 0.8291302328435669,
                "draw-right_sc_sec-itrem_ps_amp_diss": 0.15244557609398457,
                "draw-right_sc_sec-itrem_ps_amp_x_traj": 0.14820909445825728,
                "draw-right_sc_sec-itrem_ps_amp_y_traj": 207.09014473960204,
                "draw-right_sc_sec-corner": -2.5639249893428135,
                "draw-right_sc_sec-axes_over": 6.7488564631521895,
                "draw-right_sc_sec-raw_pacman_score": 0.3622350674373796,
            },
        ),
        (
            "infinity-right",
            {
                "draw-right_inf_first-dur_acc-mean": 5.8428439783045616e-05,
                "draw-right_inf_first-dur_acc-median": 7.51296984634137e-05,
                "draw-right_inf_first-sim-mean": 8.974804922475677,
                "draw-right_inf_first-sim-median": 6.979714543015415,
                "draw-right_inf_first-press-mean": 1.829741358757019,
                "draw-right_inf_first-press-cv": 0.4135418236255646,
                "draw-right_inf_first-user_dur": 1907.0,
                "draw-right_inf_first-smooth": -2.2437631429562876,
                "draw-right_inf_first-rt": 311.0,
                "draw-right_inf_first-speed-mean": 740.8425619716963,
                "draw-right_inf_first-speed-std": 174.96914745091593,
                "draw-right_inf_first-speed-median": 725.4889993106617,
                "draw-right_inf_first-speed-min": 0.0,
                "draw-right_inf_first-speed-max": 1171.2709154401507,
                "draw-right_inf_first-speed-q95": 1012.3133834670571,
                "draw-right_inf_first-speed-cv": 0.23617588463768713,
                "draw-right_inf_first-c_ratio": 0.965247226339179,
                "draw-right_inf_first-raw_pacman_score": 0.586,
            },
        ),
        (
            "infinity-right-02",
            {
                "draw-right_inf_sec-dur_acc-mean": 4.174007408612183e-05,
                "draw-right_inf_sec-dur_acc-median": 4.392339811507962e-05,
                "draw-right_inf_sec-sim-mean": 11.231969849072678,
                "draw-right_inf_sec-sim-median": 10.673656268694419,
                "draw-right_inf_sec-press-mean": 2.179748058319092,
                "draw-right_inf_sec-press-cv": 0.3925800621509552,
                "draw-right_inf_sec-user_dur": 2133.0,
                "draw-right_inf_sec-smooth": -2.24783049772932,
                "draw-right_inf_sec-rt": 188.0,
                "draw-right_inf_sec-speed-mean": 656.2139157132024,
                "draw-right_inf_sec-speed-std": 184.91862838826256,
                "draw-right_inf_sec-speed-median": 654.015064239502,
                "draw-right_inf_sec-speed-min": 0.0,
                "draw-right_inf_sec-speed-max": 1093.6998128890991,
                "draw-right_inf_sec-speed-q95": 960.0566134733311,
                "draw-right_inf_sec-speed-cv": 0.28179626179869227,
                "draw-right_inf_sec-c_ratio": 0.9506422101113992,
                "draw-right_inf_sec-raw_pacman_score": 0.401,
            },
        ),
        (
            "spiral-right",
            {
                "draw-right_spi_first-dur_acc-mean": 9.000798266622234e-05,
                "draw-right_spi_first-dur_acc-median": 0.00011177971951531323,
                "draw-right_spi_first-sim-mean": 4.401793060287793,
                "draw-right_spi_first-sim-median": 3.5444400396478417,
                "draw-right_spi_first-press-mean": 1.6639434099197388,
                "draw-right_spi_first-press-cv": 0.4043589234352112,
                "draw-right_spi_first-user_dur": 2524.0,
                "draw-right_spi_first-smooth": -2.110820346530645,
                "draw-right_spi_first-rt": 296.0,
                "draw-right_spi_first-speed-mean": 769.6063602682623,
                "draw-right_spi_first-speed-std": 198.96339521641937,
                "draw-right_spi_first-speed-median": 776.5834033489227,
                "draw-right_spi_first-speed-min": 0.0,
                "draw-right_spi_first-speed-max": 1202.6730855305989,
                "draw-right_spi_first-speed-q95": 1020.1837658882141,
                "draw-right_spi_first-speed-cv": 0.25852618362855856,
                "draw-right_spi_first-c_ratio": 0.9868183871486966,
                "draw-right_spi_first-cross": 13.0,
                "draw-right_spi_first-cross_per_sec": 5.17104216388226,
                "draw-right_spi_first-cross_freq-mean": 12.464047460392345,
                "draw-right_spi_first-cross_freq-std": 9.639343208484469,
                "draw-right_spi_first-cross_freq-cv": 0.7733718311901422,
                "draw-right_spi_first-raw_pacman_score": 0.8575096277278562,
            },
        ),
        (
            "spiral-right-02",
            {
                "draw-right_spi_sec-dur_acc-mean": 4.3181775507409e-05,
                "draw-right_spi_sec-dur_acc-median": 4.406113012725701e-05,
                "draw-right_spi_sec-sim-mean": 9.999100873334221,
                "draw-right_spi_sec-sim-median": 9.799542770264745,
                "draw-right_spi_sec-press-mean": 1.4327380657196045,
                "draw-right_spi_sec-press-cv": 0.4085606634616852,
                "draw-right_spi_sec-user_dur": 2316.0,
                "draw-right_spi_sec-smooth": -2.0865597708317996,
                "draw-right_spi_sec-rt": 121.0,
                "draw-right_spi_sec-speed-mean": 865.2573237496703,
                "draw-right_spi_sec-speed-std": 207.65615009544888,
                "draw-right_spi_sec-speed-median": 857.47344153268,
                "draw-right_spi_sec-speed-min": 0.0,
                "draw-right_spi_sec-speed-max": 1520.5577850341795,
                "draw-right_spi_sec-speed-q95": 1156.1517688963147,
                "draw-right_spi_sec-speed-cv": 0.23999351914821399,
                "draw-right_spi_sec-c_ratio": 1.0096490741548805,
                "draw-right_spi_sec-cross": 8.0,
                "draw-right_spi_sec-cross_per_sec": 3.4797738147020443,
                "draw-right_spi_sec-cross_freq-mean": 4.690346957022657,
                "draw-right_spi_sec-cross_freq-std": 2.787474760333405,
                "draw-right_spi_sec-cross_freq-cv": 0.5943003334028067,
                "draw-right_spi_sec-raw_pacman_score": 0.4184852374839538,
            },
        ),
        (
            "square_counter_clock-left",
            {
                "draw-left_scc_first-dur_acc-mean": 4.796022723838809e-05,
                "draw-left_scc_first-dur_acc-median": 6.01641895674781e-05,
                "draw-left_scc_first-sim-mean": 9.666485921285847,
                "draw-left_scc_first-sim-median": 7.705694445723113,
                "draw-left_scc_first-press-mean": 0.6208332777023315,
                "draw-left_scc_first-press-cv": 0.24515151977539062,
                "draw-left_scc_first-user_dur": 2157.0,
                "draw-left_scc_first-smooth": -2.6895313847548756,
                "draw-left_scc_first-rt": 579.0,
                "draw-left_scc_first-speed-mean": 700.7668192426084,
                "draw-left_scc_first-speed-std": 417.95821526989687,
                "draw-left_scc_first-speed-median": 623.1563431876046,
                "draw-left_scc_first-speed-min": 19.610236672794116,
                "draw-left_scc_first-speed-max": 1667.968988418579,
                "draw-left_scc_first-speed-q95": 1521.4111776912912,
                "draw-left_scc_first-speed-cv": 0.5964298020297647,
                "draw-left_scc_first-c_ratio": 0.9664718754651813,
                "draw-left_scc_first-itrem_ps_ene_diss": 1.9837373342140434,
                "draw-left_scc_first-itrem_ps_ene_x_traj": 6.09768548883777,
                "draw-left_scc_first-itrem_ps_ene_y_traj": 1085.5935586167411,
                "draw-left_scc_first-itrem_ps_peak_diss": 3.333466672000213,
                "draw-left_scc_first-itrem_ps_peak_x_traj": 3.333466672000213,
                "draw-left_scc_first-itrem_ps_peak_y_traj": 3.333466672000213,
                "draw-left_scc_first-itrem_ps_ent_diss": 1.8804860137869006,
                "draw-left_scc_first-itrem_ps_ent_x_traj": 1.2452525360124833,
                "draw-left_scc_first-itrem_ps_ent_y_traj": 1.006406029037572,
                "draw-left_scc_first-itrem_ps_amp_diss": 1.190194790832405,
                "draw-left_scc_first-itrem_ps_amp_x_traj": 3.65846494885093,
                "draw-left_scc_first-itrem_ps_amp_y_traj": 651.330080924638,
                "draw-left_scc_first-corner": -16.257357057047823,
                "draw-left_scc_first-axes_over": -0.5329087578227151,
                "draw-left_scc_first-raw_pacman_score": 0.5096339113680154,
            },
        ),
        (
            "square_counter_clock-left-02",
            {
                "draw-left_scc_sec-dur_acc-mean": 4.79735398509855e-05,
                "draw-left_scc_sec-dur_acc-median": 5.505940481584662e-05,
                "draw-left_scc_sec-sim-mean": 9.035467753504381,
                "draw-left_scc_sec-sim-median": 7.872649074119341,
                "draw-left_scc_sec-press-mean": 0.5714285969734192,
                "draw-left_scc_sec-press-cv": 0.5420739650726318,
                "draw-left_scc_sec-user_dur": 2307.0,
                "draw-left_scc_sec-smooth": -2.5262265312687915,
                "draw-left_scc_sec-rt": 195.0,
                "draw-left_scc_sec-speed-mean": 679.8335019881044,
                "draw-left_scc_sec-speed-std": 382.4641631803236,
                "draw-left_scc_sec-speed-median": 568.6268525965073,
                "draw-left_scc_sec-speed-min": 0.0,
                "draw-left_scc_sec-speed-max": 1712.4106089274087,
                "draw-left_scc_sec-speed-q95": 1462.4986648559568,
                "draw-left_scc_sec-speed-cv": 0.5625850477533776,
                "draw-left_scc_sec-c_ratio": 1.006220812457585,
                "draw-left_scc_sec-itrem_ps_ene_diss": 0.18759668493310636,
                "draw-left_scc_sec-itrem_ps_ene_x_traj": 9.62158172392461,
                "draw-left_scc_sec-itrem_ps_ene_y_traj": 1152.0837186656997,
                "draw-left_scc_sec-itrem_ps_peak_diss": 7.05910589364751,
                "draw-left_scc_sec-itrem_ps_peak_x_traj": 3.529552946823755,
                "draw-left_scc_sec-itrem_ps_peak_y_traj": 3.529552946823755,
                "draw-left_scc_sec-itrem_ps_ent_diss": 2.2632946772917744,
                "draw-left_scc_sec-itrem_ps_ent_x_traj": 1.2423343239649807,
                "draw-left_scc_sec-itrem_ps_ent_y_traj": 0.5434057679068282,
                "draw-left_scc_sec-itrem_ps_amp_diss": 0.26360782482745887,
                "draw-left_scc_sec-itrem_ps_amp_x_traj": 5.452011554371537,
                "draw-left_scc_sec-itrem_ps_amp_y_traj": 652.8213266796067,
                "draw-left_scc_sec-corner": 18.59651145658366,
                "draw-left_scc_sec-axes_over": 11.516697794338148,
                "draw-left_scc_sec-raw_pacman_score": 0.5192678227360308,
            },
        ),
        (
            "square_clock-left",
            {
                "draw-left_sc_first-dur_acc-mean": 4.796272519546591e-05,
                "draw-left_sc_first-dur_acc-median": 4.944918458526755e-05,
                "draw-left_sc_first-sim-mean": 9.811540803327915,
                "draw-left_sc_first-sim-median": 9.5166025333069,
                "draw-left_sc_first-press-mean": 0.96217942237854,
                "draw-left_sc_first-press-cv": 0.4250722825527191,
                "draw-left_sc_first-user_dur": 2125.0,
                "draw-left_sc_first-smooth": -1.6541240769400725,
                "draw-left_sc_first-rt": 344.0,
                "draw-left_sc_first-speed-mean": 719.3787697368397,
                "draw-left_sc_first-speed-std": 449.3305332535327,
                "draw-left_sc_first-speed-median": 608.1598506254307,
                "draw-left_sc_first-speed-min": 0.0,
                "draw-left_sc_first-speed-max": 1937.6119375228882,
                "draw-left_sc_first-speed-q95": 1749.621851303998,
                "draw-left_sc_first-speed-cv": 0.6246091101880933,
                "draw-left_sc_first-c_ratio": 0.9904573709248382,
                "draw-left_sc_first-itrem_ps_ene_diss": 20.658790739965337,
                "draw-left_sc_first-itrem_ps_ene_x_traj": 24.373051824182866,
                "draw-left_sc_first-itrem_ps_ene_y_traj": 734.4459461498769,
                "draw-left_sc_first-itrem_ps_peak_diss": 4.6155692381541416,
                "draw-left_sc_first-itrem_ps_peak_x_traj": 4.6155692381541416,
                "draw-left_sc_first-itrem_ps_peak_y_traj": 4.6155692381541416,
                "draw-left_sc_first-itrem_ps_ent_diss": 0.28103312971316646,
                "draw-left_sc_first-itrem_ps_ent_x_traj": 0.31740232782411926,
                "draw-left_sc_first-itrem_ps_ent_y_traj": 0.7932755161390063,
                "draw-left_sc_first-itrem_ps_amp_diss": 8.95178456827882,
                "draw-left_sc_first-itrem_ps_amp_x_traj": 10.561233324247622,
                "draw-left_sc_first-itrem_ps_amp_y_traj": 318.2471796018801,
                "draw-left_sc_first-corner": 2.340849290068158,
                "draw-left_sc_first-axes_over": 9.724545107061338,
                "draw-left_sc_first-raw_pacman_score": 0.28709055876685935,
            },
        ),
        (
            "square_clock-left-02",
            {
                "draw-left_sc_sec-dur_acc-mean": 5.651797506753375e-05,
                "draw-left_sc_sec-dur_acc-median": 6.146695135137353e-05,
                "draw-left_sc_sec-sim-mean": 8.049811642075541,
                "draw-left_sc_sec-sim-median": 7.401685681211223,
                "draw-left_sc_sec-press-mean": 1.0560777187347412,
                "draw-left_sc_sec-press-cv": 0.4368475675582886,
                "draw-left_sc_sec-user_dur": 2198.0,
                "draw-left_sc_sec-smooth": -1.671406363108141,
                "draw-left_sc_sec-rt": 88.0,
                "draw-left_sc_sec-speed-mean": 717.4865668148818,
                "draw-left_sc_sec-speed-std": 551.7240680238751,
                "draw-left_sc_sec-speed-median": 607.3913276195526,
                "draw-left_sc_sec-speed-min": 0.0,
                "draw-left_sc_sec-speed-max": 2312.5,
                "draw-left_sc_sec-speed-q95": 2056.3744678216817,
                "draw-left_sc_sec-speed-cv": 0.7689678016873939,
                "draw-left_sc_sec-c_ratio": 1.008917784661532,
                "draw-left_sc_sec-itrem_ps_ene_diss": 0.0,
                "draw-left_sc_sec-itrem_ps_ene_x_traj": 0.0,
                "draw-left_sc_sec-itrem_ps_ene_y_traj": 0.0,
                "draw-left_sc_sec-itrem_ps_peak_diss": 8.571771442286263,
                "draw-left_sc_sec-itrem_ps_peak_x_traj": 8.571771442286263,
                "draw-left_sc_sec-itrem_ps_peak_y_traj": 8.571771442286263,
                "draw-left_sc_sec-itrem_ps_ent_diss": 1.1855741114956662,
                "draw-left_sc_sec-itrem_ps_ent_x_traj": 1.1898276991320587,
                "draw-left_sc_sec-itrem_ps_ent_y_traj": 0.8822160191720476,
                "draw-left_sc_sec-itrem_ps_amp_diss": 0.9632978102858575,
                "draw-left_sc_sec-itrem_ps_amp_x_traj": 0.969757093073007,
                "draw-left_sc_sec-itrem_ps_amp_y_traj": 357.67462109246804,
                "draw-left_sc_sec-corner": 7.471485170119777,
                "draw-left_sc_sec-axes_over": 9.51377018876152,
                "draw-left_sc_sec-raw_pacman_score": 0.5308285163776493,
            },
        ),
        (
            "infinity-left",
            {
                "draw-left_inf_first-dur_acc-mean": 3.829319758158458e-05,
                "draw-left_inf_first-dur_acc-median": 4.453494836004561e-05,
                "draw-left_inf_first-sim-mean": 11.663375928810233,
                "draw-left_inf_first-sim-median": 10.028707236829732,
                "draw-left_inf_first-press-mean": 0.7807097434997559,
                "draw-left_inf_first-press-cv": 0.38773107528686523,
                "draw-left_inf_first-user_dur": 2239.0,
                "draw-left_inf_first-smooth": -2.264604305962189,
                "draw-left_inf_first-rt": 181.0,
                "draw-left_inf_first-speed-mean": 645.0462643904962,
                "draw-left_inf_first-speed-std": 179.3900466149476,
                "draw-left_inf_first-speed-median": 647.2069515901453,
                "draw-left_inf_first-speed-min": 0.0,
                "draw-left_inf_first-speed-max": 1050.998568534851,
                "draw-left_inf_first-speed-q95": 925.7605493068695,
                "draw-left_inf_first-speed-cv": 0.27810415549720785,
                "draw-left_inf_first-c_ratio": 0.9778909172776828,
                "draw-left_inf_first-raw_pacman_score": 0.408,
            },
        ),
        (
            "infinity-left-02",
            {
                "draw-left_inf_sec-dur_acc-mean": 6.548676142557583e-05,
                "draw-left_inf_sec-dur_acc-median": 7.527767493112353e-05,
                "draw-left_inf_sec-sim-mean": 6.525752962086865,
                "draw-left_inf_sec-sim-median": 5.676987602784467,
                "draw-left_inf_sec-press-mean": 0.9650821089744568,
                "draw-left_inf_sec-press-cv": 0.5000325441360474,
                "draw-left_inf_sec-user_dur": 2340.0,
                "draw-left_inf_sec-smooth": -2.385696587888054,
                "draw-left_inf_sec-rt": 213.0,
                "draw-left_inf_sec-speed-mean": 627.6823289924221,
                "draw-left_inf_sec-speed-std": 166.3679102992792,
                "draw-left_inf_sec-speed-median": 631.4205842859604,
                "draw-left_inf_sec-speed-min": 0.0,
                "draw-left_inf_sec-speed-max": 1182.7593644460042,
                "draw-left_inf_sec-speed-q95": 861.2508773803711,
                "draw-left_inf_sec-speed-cv": 0.2650511295520122,
                "draw-left_inf_sec-c_ratio": 0.9997710564333739,
                "draw-left_inf_sec-raw_pacman_score": 0.666,
            },
        ),
        (
            "spiral-left",
            {
                "draw-left_spi_first-dur_acc-mean": 4.756669022368151e-05,
                "draw-left_spi_first-dur_acc-median": 5.866295227730068e-05,
                "draw-left_spi_first-sim-mean": 7.212046321252353,
                "draw-left_spi_first-sim-median": 5.847867519865601,
                "draw-left_spi_first-press-mean": 0.9166666269302368,
                "draw-left_spi_first-press-cv": 0.416151225566864,
                "draw-left_spi_first-user_dur": 2915.0,
                "draw-left_spi_first-smooth": -2.049361968577463,
                "draw-left_spi_first-rt": 271.0,
                "draw-left_spi_first-speed-mean": 657.8616223770069,
                "draw-left_spi_first-speed-std": 157.25727146577864,
                "draw-left_spi_first-speed-median": 647.355416241814,
                "draw-left_spi_first-speed-min": 0.0,
                "draw-left_spi_first-speed-max": 1362.8400655893179,
                "draw-left_spi_first-speed-q95": 925.6473821752211,
                "draw-left_spi_first-speed-cv": 0.23904308461948512,
                "draw-left_spi_first-c_ratio": 0.9652024842346388,
                "draw-left_spi_first-cross": 9.0,
                "draw-left_spi_first-cross_per_sec": 3.105590062111801,
                "draw-left_spi_first-cross_freq-mean": 4.359583762941149,
                "draw-left_spi_first-cross_freq-std": 2.47239037898574,
                "draw-left_spi_first-cross_freq-cv": 0.5671161545288825,
                "draw-left_spi_first-raw_pacman_score": 0.6700898587933247,
            },
        ),
        (
            "spiral-left-02",
            {
                "draw-left_spi_sec-dur_acc-mean": 3.997969912792753e-05,
                "draw-left_spi_sec-dur_acc-median": 4.45951238170584e-05,
                "draw-left_spi_sec-sim-mean": 9.01683290835422,
                "draw-left_spi_sec-sim-median": 8.083625201751401,
                "draw-left_spi_sec-press-mean": 0.7658729553222656,
                "draw-left_spi_sec-press-cv": 0.515119731426239,
                "draw-left_spi_sec-user_dur": 2774.0,
                "draw-left_spi_sec-smooth": -2.105913524921934,
                "draw-left_spi_sec-rt": 128.0,
                "draw-left_spi_sec-speed-mean": 674.0391426381944,
                "draw-left_spi_sec-speed-std": 204.32050164119926,
                "draw-left_spi_sec-speed-median": 665.8004872939165,
                "draw-left_spi_sec-speed-min": 0.0,
                "draw-left_spi_sec-speed-max": 1182.1144648960658,
                "draw-left_spi_sec-speed-q95": 990.4006958007811,
                "draw-left_spi_sec-speed-cv": 0.3031285406385855,
                "draw-left_spi_sec-c_ratio": 0.9452660448357265,
                "draw-left_spi_sec-cross": 5.0,
                "draw-left_spi_sec-cross_per_sec": 1.8315018315018314,
                "draw-left_spi_sec-cross_freq-mean": 3.571392494794924,
                "draw-left_spi_sec-cross_freq-std": 2.2519755059144786,
                "draw-left_spi_sec-cross_freq-cv": 0.6305595112261082,
                "draw-left_spi_sec-raw_pacman_score": 0.4813863928112965,
            },
        ),
        (
            "left-all",
            {
                "draw-left-acc_sig_ene": 0.5345981,
                "draw-left-acc_ps_ene_x": 0.00019211073031710768,
                "draw-left-acc_ps_ene_y": 4.9899592756766364e-05,
                "draw-left-acc_ps_ene_z": 0.00027198949729578814,
                "draw-left-acc_ps_ene_xyz": 9.315513409990928e-05,
                "draw-left-acc_ps_peak_x": 0.15625,
                "draw-left-acc_ps_peak_y": 0.3125,
                "draw-left-acc_ps_peak_z": 0.3125,
                "draw-left-acc_ps_peak_xyz": 0.3125,
                "draw-left-acc_ps_ent_x": 6.005536521789103,
                "draw-left-acc_ps_ent_y": 5.882338670264282,
                "draw-left-acc_ps_ent_z": 5.818906554311909,
                "draw-left-acc_ps_ent_xyz": 4.944414540951332,
                "draw-left-acc_ps_amp_x": 2.2232106857700273e-05,
                "draw-left-acc_ps_amp_y": 1.021672142087482e-05,
                "draw-left-acc_ps_amp_z": 9.99491021502763e-05,
                "draw-left-acc_ps_amp_xyz": 0.00010004831710830331,
                "draw-left-gyr_sig_ene": 1.5336087,
                "draw-left-gyr_ps_ene_x": 0.001064549080842525,
                "draw-left-gyr_ps_ene_y": 0.0007276217445717492,
                "draw-left-gyr_ps_ene_z": 0.000577464406796846,
                "draw-left-gyr_ps_ene_xyz": 0.0006846297704399262,
                "draw-left-gyr_ps_peak_x": 0.3125,
                "draw-left-gyr_ps_peak_y": 0.3125,
                "draw-left-gyr_ps_peak_z": 1.015625,
                "draw-left-gyr_ps_peak_xyz": 0.3125,
                "draw-left-gyr_ps_ent_x": 5.043816995308092,
                "draw-left-gyr_ps_ent_y": 5.817817941364398,
                "draw-left-gyr_ps_ent_z": 5.6690061711122155,
                "draw-left-gyr_ps_ent_xyz": 4.8221132474110595,
                "draw-left-gyr_ps_amp_x": 0.001086545642465353,
                "draw-left-gyr_ps_amp_y": 0.0005311050917953253,
                "draw-left-gyr_ps_amp_z": 0.00018349679885432124,
                "draw-left-gyr_ps_amp_xyz": 0.0011522744316607714,
            },
        ),
        (
            "right-all",
            {
                "draw-right-acc_sig_ene": 0.5494641,
                "draw-right-acc_ps_ene_x": 0.00019662130872843964,
                "draw-right-acc_ps_ene_y": 0.0001080299358710568,
                "draw-right-acc_ps_ene_z": 0.000235807236290686,
                "draw-right-acc_ps_ene_xyz": 0.00014518453659739805,
                "draw-right-acc_ps_peak_x": 0.390625,
                "draw-right-acc_ps_peak_y": 0.390625,
                "draw-right-acc_ps_peak_z": 0.390625,
                "draw-right-acc_ps_peak_xyz": 0.390625,
                "draw-right-acc_ps_ent_x": 5.821881314507291,
                "draw-right-acc_ps_ent_y": 5.778812261530092,
                "draw-right-acc_ps_ent_z": 5.905741809572408,
                "draw-right-acc_ps_ent_xyz": 5.394053018899435,
                "draw-right-acc_ps_amp_x": 5.023233825340867e-05,
                "draw-right-acc_ps_amp_y": 4.195844303467311e-05,
                "draw-right-acc_ps_amp_z": 9.071591193787754e-05,
                "draw-right-acc_ps_amp_xyz": 8.2131773524452e-05,
                "draw-right-gyr_sig_ene": 3.8941433,
                "draw-right-gyr_ps_ene_x": 0.005255818428793191,
                "draw-right-gyr_ps_ene_y": 0.010284893235450454,
                "draw-right-gyr_ps_ene_z": 0.0011059606457752835,
                "draw-right-gyr_ps_ene_xyz": 0.005606818721446416,
                "draw-right-gyr_ps_peak_x": 0.3125,
                "draw-right-gyr_ps_peak_y": 0.703125,
                "draw-right-gyr_ps_peak_z": 0.703125,
                "draw-right-gyr_ps_peak_xyz": 0.3125,
                "draw-right-gyr_ps_ent_x": 5.035798235744754,
                "draw-right-gyr_ps_ent_y": 5.682501151984273,
                "draw-right-gyr_ps_ent_z": 5.57331173362358,
                "draw-right-gyr_ps_ent_xyz": 5.387239599527969,
                "draw-right-gyr_ps_amp_x": 0.003255309537053108,
                "draw-right-gyr_ps_amp_y": 0.0035879213828593493,
                "draw-right-gyr_ps_amp_z": 0.0005053733475506306,
                "draw-right-gyr_ps_amp_xyz": 0.003069657366722822,
            },
        ),
        (
            "all_levels",
            {
                "draw-press-mean": 1.2964386,
                "draw-press-std": 0.78505987,
                "draw-press-median": 1.125,
                "draw-press-min": 0.0,
                "draw-press-max": 3.0833333,
                "draw-press-cv": 0.6055511,
                "draw-rt-mean": 247.5,
                "draw-rt-std": 141.76600438751174,
                "draw-rt-median": 204.0,
                "draw-rt-min": 88.0,
                "draw-rt-max": 579.0,
            },
        ),
    ],
)
def test_draw_process_new_format(
    example_reading_processed_draw_new_format, level, expected
):
    """Unit test to ensure the measures are well computed on new format."""
    assert_level_values(example_reading_processed_draw_new_format, level, expected)


def test_draw_process_new_format_wo_end_zone_are_invalid(
    example_reading_processed_draw_nf_no_end_zone,
):
    """Test the right hand levels without end_zone are invalid."""
    for it, level in enumerate(example_reading_processed_draw_nf_no_end_zone.levels):
        if it <= 7:
            assert len(level.get_flags()) == 1
            assert (
                level.get_flags()[0].id == "draw-technical-deviation-complete_drawing"
            )
        else:
            assert len(level.get_flags()) == 0


def test_draw_process_summary_measures(example_reading_processed_draw):
    """Test computing summary measures."""
    ms = example_reading_processed_draw.measure_set
    assert_measure_value(ms, "draw-right-sim-mean", 6.162360142536366)
    assert_measure_value(ms, "draw-left-sim-mean", 6.904851328371154)
    assert_measure_value(ms, "draw-sim-mean", 6.533605735453761)
    assert_measure_value(ms, "draw-dur_acc-mean", 5.888455548511744e-05)
    assert_measure_value(ms, "draw-smooth-mean", -2.223039246494186)
    assert_measure_value(ms, "draw-user_dur-mean", 2912.875)
    assert_measure_value(ms, "draw-cross_per_sec-mean", 2.657783190303219)
    assert_measure_value(ms, "draw-cross-mean", 10.25)
    assert_measure_value(ms, "draw-corner-mean", -0.9198598857182514)
    assert_measure_value(ms, "draw-axes_over-mean", 3.6695091841475875)


def test_draw_process_summary_ratio_flag(
    example_reading_processed_draw_distance_ratio,
):
    """Test computing summary measures with only invalid ratio."""
    inval_spi = example_reading_processed_draw_distance_ratio.get_level(
        "spiral-right"
    ).get_flags()
    assert len(inval_spi) > 0
    assert inval_spi[0].id == "draw-behavioral-deviation-dist_thresh"

    inval_inf = example_reading_processed_draw_distance_ratio.get_level(
        "infinity-right"
    ).get_flags()
    assert len(inval_inf) > 0
    assert inval_inf[0].id == "draw-behavioral-deviation-dist_thresh"


def test_get_reference_path(example_level_draw_sccr, height_screen):
    """Test the good computation of reference path data frame."""
    ref = get_reference_path(example_level_draw_sccr, height_screen)
    path_model = ref[["x", "y"]].dropna().to_numpy()
    assert list(ref.columns) == ["x", "y"]
    assert path_model.size == 2076


def test_get_user_path(
    example_level_draw_sccr, example_data_draw_sccr_screen, height_screen
):
    """Test the good computation of user path from 'screen' data frame."""
    data = get_user_path(
        example_data_draw_sccr_screen,
        get_reference_path(example_level_draw_sccr, height_screen),
        height_screen,
    )
    path_user = data[["x", "y"]]
    assert list(data.columns) == [
        "x",
        "y",
        "touchAction",
        "tsTouch",
        "pressure",
        "isValidArea",
    ]
    assert path_user.size == 378


def test_get_up_sampling_paths(example_data_draw_sccr_screen_path):
    """Test the up-sampling method."""
    paths_up = get_valid_up_sampled_path(example_data_draw_sccr_screen_path)
    assert isinstance(paths_up, pd.DataFrame)
    assert "y" in paths_up.columns
    assert "x" in paths_up.columns
    assert example_data_draw_sccr_screen_path["x"][0] == paths_up["x"][0]
    path_size = len(example_data_draw_sccr_screen_path["x"].dropna()) - 1
    assert example_data_draw_sccr_screen_path["x"][path_size] == 331.0
    path_size_up = len(paths_up["x"].dropna()) - 1
    assert paths_up["x"][path_size_up] == pytest.approx(331.0)


def test_get_max_dist_corner(example_data_draw_sccl_shape):
    """Test the good behavior of the max distance corner computation."""
    data = example_data_draw_sccl_shape.up_sampled_data_without_overshoot
    ref = example_data_draw_sccl_shape.reference_without_head
    attrib = example_data_draw_sccl_shape.up_sampled_valid_no_overshoot_matches

    # Undershoot example
    corner = (319, 123)
    assert get_max_dist_corner(data, attrib, ref, corner) == -8.350403467168107
    # Overshoot example
    corner = (56, 123)
    assert get_max_dist_corner(data, attrib, ref, corner) == 6.162819611170232


def test_extract_distance_axis_sc(example_data_draw_scl_shape):
    """Test the good behavior of the distance perp. axis computation (sc)."""
    data = example_data_draw_scl_shape.up_sampled_data_without_overshoot
    ref = example_data_draw_scl_shape.reference_without_head
    res = extract_distance_axis_sc(data, ref)
    assert_almost_equal(
        res, [0.06080267174212395, 5.445709956915735, 14.059580225082698], decimal=13
    )


def test_extract_distance_axis_scc(example_data_draw_sccl_shape):
    """Test the good behavior of the distance perp. axis computation (scc)."""
    data = example_data_draw_sccl_shape.up_sampled_data_without_overshoot
    ref = example_data_draw_sccl_shape.reference_without_head
    res = extract_distance_axis_scc(data, ref)
    # With negative example
    assert_almost_equal(
        res, [9.722733232473843, 1.369466970178891, -1.9646686736996344], decimal=13
    )
