"""Tests for :mod:`bal.processing.typing`."""

import pytest  # type: ignore

from dispel.processing.core import StopProcessingError
from dispel.providers.bdh.tasks.typing.steps import process_typing
from tests.processing.helper import assert_level_values


@pytest.mark.parametrize(
    "level,expected_measures",
    [
        (
            "typing",
            {
                "tt-rt-mean": 0.876375,
                "tt-rt-std": 0.14481194702095543,
                "tt-rt_fc-mean": 0.898375,
                "tt-rt_fc-std": 0.17513768776999242,
                "tt-rt_fc_slope": 0.006477072208919195,
                "tt-rt_slope": 0.002132188030700065,
                "tt-n_correct_words": 32,
                "tt-word_duration-mean": 1.8082727272727273,
                "tt-word_duration-std": 0.4314178856346299,
                "tt-word_duration-median": 1.747,
                "tt-word_duration-min": 0.462,
                "tt-word_duration-max": 2.9250000000000003,
                "tt-word_duration-0-mean": 1.5836666666666668,
                "tt-word_duration-0-median": 1.6660000000000001,
                "tt-word_duration-0-std": 0.15844978173962046,
                "tt-word_duration-1-mean": 1.4816666666666667,
                "tt-word_duration-1-median": 1.411,
                "tt-word_duration-1-std": 0.24671103204626527,
                "tt-word_duration-2-mean": 1.869,
                "tt-word_duration-2-median": 1.6920000000000002,
                "tt-word_duration-2-std": 0.371589827632566,
                "tt-word_duration-3-mean": 1.6320000000000003,
                "tt-word_duration-3-median": 1.6520000000000001,
                "tt-word_duration-3-std": 0.04915282290977807,
                "tt-word_duration-4-mean": 1.619,
                "tt-word_duration-4-median": 1.639,
                "tt-word_duration-4-std": 0.13908270920570967,
                "tt-word_duration-5-mean": 1.8490000000000002,
                "tt-word_duration-5-median": 1.6740000000000002,
                "tt-word_duration-5-std": 0.565198195326206,
                "tt-word_duration-6-mean": 2.134666666666667,
                "tt-word_duration-6-median": 2.0620000000000003,
                "tt-word_duration-6-std": 0.2355617399607443,
                "tt-word_duration-7-mean": 1.8603333333333334,
                "tt-word_duration-7-median": 1.8410000000000002,
                "tt-word_duration-7-std": 0.07489548272982374,
                "tt-word_duration-8-mean": 2.0253333333333337,
                "tt-word_duration-8-median": 2.1420000000000003,
                "tt-word_duration-8-std": 0.3061524674624285,
                "tt-word_duration-9-mean": 2.3143333333333334,
                "tt-word_duration-9-median": 2.2070000000000003,
                "tt-word_duration-9-std": 0.5647028717240008,
                "tt-word_duration-10-mean": 2.052,
                "tt-word_duration-10-median": 2.052,
                "tt-word_duration-10-std": 0.1456639969244289,
                "tt-word_duration_slope-mean": 0.046833333333333324,
                "tt-word_duration_slope-std": 0.25476884948255873,
                "tt-word_duration_slope-median": 0.07599999999999996,
                "tt-word_duration_slope-min": -0.27433333333333354,
                "tt-word_duration_slope-max": 0.3873333333333333,
                "tt-word_duration-last_three-mean": 1.9716666666666667,
                "tt-word_duration-last_three-median": 1.949,
                "tt-word_duration-last_three-std": 0.17311653108046426,
                "tt-reaction_duration-mean": 0.05,
                "tt-reaction_duration-median": 0.05,
                "tt-reaction_duration-min": 0.05,
                "tt-reaction_duration-max": 0.05,
                "tt-words_typed_without_error": 31,
                "tt-words_typed_with_error": 1,
                "tt-consecutive_words_typed_without_error": 27,
                "tt-consecutive_words_typed_with_error": 1,
                "tt-n_incorrect_words": 1,
                "tt-ratio_letters_typed_len-mean": 1.0182291666666665,
                "tt-ratio_letters_typed_len-std": 0.07250749250618206,
                "tt-ratio_letters_typed_len-median": 1.0,
                "tt-ratio_letters_typed_len-min": 1.0,
                "tt-ratio_letters_typed_len-max": 1.3333333333333333,
                "tt-correct_letters": 199,
                "tt-letter_intervals-mean": 0.2936281407035176,
                "tt-letter_intervals-std": 0.27322949323193146,
                "tt-letter_intervals-median": 0.19,
                "tt-letter_intervals-min": 0.05,
                "tt-letter_intervals-max": 1.2870000000000001,
                "tt-letter_intervals_correct-mean": 0.2936281407035176,
                "tt-letter_intervals_correct-std": 0.27322949323193146,
                "tt-letter_intervals_correct-median": 0.19,
                "tt-letter_intervals_correct-min": 0.05,
                "tt-letter_intervals_correct-max": 1.2870000000000001,
                "tt-top_10_letter_intervals-mean": 0.9562999999999999,
                "tt-top_10_letter_intervals-std": 0.12225172477526165,
                "tt-top_10_letter_intervals-median": 0.914,
                "tt-top_10_letter_intervals-min": 0.8210000000000001,
                "tt-top_10_letter_intervals-max": 1.2870000000000001,
                "tt-top_10_letter_intervals-iqr": 0.12150000000000005,
                "tt-top_10_letter_intervals_correct-mean": 0.9562999999999999,
                "tt-top_10_letter_intervals_correct-std": 0.12225172477526165,
                "tt-top_10_letter_intervals_correct-median": 0.914,
                "tt-top_10_letter_intervals_correct-min": 0.8210000000000001,
                "tt-top_10_letter_intervals_correct-max": 1.2870000000000001,
                "tt-top_10_letter_intervals_correct-iqr": 0.12150000000000005,
                "tt-letter_interval_max_dev-mean": 0.33867123461756843,
                "tt-letter_interval_max_dev-std": 0.151504494525427,
                "tt-letter_interval_max_dev-median": 0.30248168836805556,
                "tt-letter_interval_max_dev-min": 0.0025334444444444423,
                "tt-letter_interval_max_dev-max": 0.6822760000000001,
                "tt-letter_interval_max_dev-iqr": 0.1273284829687502,
                "tt-interval_until_mistake-mean": 0.1808930817610063,
                "tt-interval_until_mistake-std": 0.07538421343609915,
                "tt-interval_until_mistake-median": 0.171,
                "tt-interval_until_mistake-min": 0.068,
                "tt-interval_until_mistake-max": 0.529,
                "tt-derived_interval_until_mistake-mean": 0.0013984375,
                "tt-derived_interval_until_mistake-std": 0.10670987525929515,
                "tt-derived_interval_until_mistake-median": -0.006500000000000001,
                "tt-derived_interval_until_mistake-min": -0.35400000000000004,
                "tt-derived_interval_until_mistake-max": 0.21000000000000002,
                "tt-ratio_correct_words_letters": 0.16,
                "tt-typing_speed_slope": 0.0021488577552766005,
                "tt-acc_sig_ene": 2.419199778436379,
                "tt-acc_ps_ene_x": 0.0010572064496358518,
                "tt-acc_ps_ene_y": 0.00012938865927384415,
                "tt-acc_ps_ene_z": 0.0011522141372589756,
                "tt-acc_ps_peak_x": 4.296875,
                "tt-acc_ps_peak_y": 3.3203125,
                "tt-acc_ps_peak_z": 4.6875,
                "tt-acc_ps_ent_x": 6.232753113944728,
                "tt-acc_ps_ent_y": 6.616514108846907,
                "tt-acc_ps_ent_z": 6.398850195595397,
                "tt-acc_ps_amp_x": 7.157626800776624e-05,
                "tt-acc_ps_amp_y": 6.585100259171798e-06,
                "tt-acc_ps_amp_z": 7.406251636052898e-05,
                "tt-gyr_sig_ene": 9.077839532755211,
                "tt-gyr_ps_ene_x": 0.01626863039575401,
                "tt-gyr_ps_ene_y": 0.047854771547193815,
                "tt-gyr_ps_ene_z": 0.0028019101766486847,
                "tt-gyr_ps_peak_x": 0.625,
                "tt-gyr_ps_peak_y": 1.328125,
                "tt-gyr_ps_peak_z": 1.40625,
                "tt-gyr_ps_ent_x": 6.730680812177365,
                "tt-gyr_ps_ent_y": 6.294308377389108,
                "tt-gyr_ps_ent_z": 6.563737556497722,
                "tt-gyr_ps_amp_x": 0.000940431634997175,
                "tt-gyr_ps_amp_y": 0.004187696297927186,
                "tt-gyr_ps_amp_z": 0.0001464164574925518,
            },
        )
    ],
)
def test_process_typing(example_reading_processed_typing, level, expected_measures):
    """Test: bal.processing.process_typing."""
    assert_level_values(example_reading_processed_typing, level, expected_measures)


@pytest.mark.parametrize(
    "level,expected_measures",
    [
        (
            "typing",
            {
                "tt-rt-mean": 0.1699259259259259,
                "tt-rt-std": 0.4016339632748212,
                "tt-rt_fc-mean": 1.0628461538461538,
                "tt-rt_fc-std": 0.3362141808202257,
                "tt-rt_fc_slope": 0.006976656515790682,
                "tt-rt_slope": -0.050842936694090395,
                "tt-n_correct_words": 25,
                "tt-word_duration-mean": 2.197111111111111,
                "tt-word_duration-std": 0.84470674626138,
                "tt-word_duration-median": 1.8610000000000002,
                "tt-word_duration-min": 1.081,
                "tt-word_duration-max": 4.437,
                "tt-word_duration-0-mean": 1.813,
                "tt-word_duration-0-median": 1.8820000000000001,
                "tt-word_duration-0-std": 0.21113265971895492,
                "tt-word_duration-1-mean": 2.2083333333333335,
                "tt-word_duration-1-median": 1.7870000000000001,
                "tt-word_duration-1-std": 0.7938301665553743,
                "tt-word_duration-2-mean": 1.704,
                "tt-word_duration-2-median": 1.7080000000000002,
                "tt-word_duration-2-std": 0.010583005244258372,
                "tt-word_duration-3-mean": 1.673,
                "tt-word_duration-3-median": 1.661,
                "tt-word_duration-3-std": 0.020784609690826544,
                "tt-word_duration-4-mean": 2.4486666666666665,
                "tt-word_duration-4-median": 1.981,
                "tt-word_duration-4-std": 0.9782762050327779,
                "tt-word_duration-5-mean": 1.8123333333333334,
                "tt-word_duration-5-median": 1.814,
                "tt-word_duration-5-std": 0.04952103929981017,
                "tt-word_duration-6-mean": 3.3906666666666667,
                "tt-word_duration-6-median": 3.902,
                "tt-word_duration-6-std": 0.9557747293862363,
                "tt-word_duration-7-mean": 2.0193333333333334,
                "tt-word_duration-7-median": 2.0820000000000003,
                "tt-word_duration-7-std": 0.13079118216964528,
                "tt-word_duration-8-mean": 2.596,
                "tt-word_duration-8-median": 2.596,
                "tt-word_duration_slope-mean": 0.09787500000000002,
                "tt-word_duration_slope-std": 0.9293453906695456,
                "tt-word_duration_slope-median": 0.1821666666666668,
                "tt-word_duration_slope-min": -1.3713333333333333,
                "tt-word_duration_slope-max": 1.5783333333333334,
                "tt-word_duration-last_three-mean": 2.2616666666666667,
                "tt-word_duration-last_three-median": 2.107,
                "tt-word_duration-last_three-std": 0.2898108578596276,
                "tt-reaction_duration-mean": 1.1902500000000003,
                "tt-reaction_duration-std": 0.46221378531295815,
                "tt-reaction_duration-median": 1.3650000000000002,
                "tt-reaction_duration-min": 0.518,
                "tt-reaction_duration-max": 1.5130000000000001,
                "tt-correcting_duration-mean": 0.7365000000000002,
                "tt-correcting_duration-std": 0.010606601717798222,
                "tt-correcting_duration-median": 0.7365000000000002,
                "tt-correcting_duration-min": 0.7290000000000001,
                "tt-correcting_duration-max": 0.7440000000000001,
                "tt-reacting_time-mean": 0.6535000000000001,
                "tt-reacting_time-std": 0.10677312395916869,
                "tt-reacting_time-median": 0.6535000000000001,
                "tt-reacting_time-min": 0.5780000000000001,
                "tt-reacting_time-max": 0.7290000000000001,
                "tt-words_typed_without_error": 21,
                "tt-words_typed_with_error": 4,
                "tt-consecutive_words_typed_without_error": 8,
                "tt-consecutive_words_typed_with_error": 1,
                "tt-n_incorrect_words": 2,
                "tt-sim_ratio-mean": 0.8699021370786078,
                "tt-sim_ratio-std": 0.13182165559600717,
                "tt-sim_ratio-median": 0.8888888888888888,
                "tt-sim_ratio-min": 0.25,
                "tt-sim_ratio-max": 0.9411764705882353,
                "tt-ratio_letters_typed_len-mean": 1.0764285714285713,
                "tt-ratio_letters_typed_len-std": 0.2983407915306864,
                "tt-ratio_letters_typed_len-median": 1.0,
                "tt-ratio_letters_typed_len-min": 0.75,
                "tt-ratio_letters_typed_len-max": 2.0,
                "tt-correct_letters": 144,
                "tt-letter_intervals-mean": 0.3284303797468354,
                "tt-letter_intervals-std": 0.3025346506887613,
                "tt-letter_intervals-median": 0.20400000000000001,
                "tt-letter_intervals-min": 0.062000000000000006,
                "tt-letter_intervals-max": 1.326,
                "tt-letter_intervals_correct-mean": 0.33376870748299325,
                "tt-letter_intervals_correct-std": 0.3070986648830885,
                "tt-letter_intervals_correct-median": 0.20400000000000001,
                "tt-letter_intervals_correct-min": 0.068,
                "tt-letter_intervals_correct-max": 1.326,
                "tt-letter_intervals_incorrect-mean": 0.2570909090909091,
                "tt-letter_intervals_incorrect-std": 0.23246524666945578,
                "tt-letter_intervals_incorrect-median": 0.194,
                "tt-letter_intervals_incorrect-min": 0.062000000000000006,
                "tt-letter_intervals_incorrect-max": 0.9430000000000001,
                "tt-top_10_letter_intervals-mean": 1.0415,
                "tt-top_10_letter_intervals-std": 0.09759644802279777,
                "tt-top_10_letter_intervals-median": 1.0015,
                "tt-top_10_letter_intervals-min": 0.9610000000000001,
                "tt-top_10_letter_intervals-max": 1.326,
                "tt-top_10_letter_intervals-iqr": 0.0694999999999999,
                "tt-top_10_letter_intervals_correct-mean": 1.0468666666666668,
                "tt-top_10_letter_intervals_correct-std": 0.09854793370789283,
                "tt-top_10_letter_intervals_correct-median": 1.002,
                "tt-top_10_letter_intervals_correct-min": 0.9630000000000001,
                "tt-top_10_letter_intervals_correct-max": 1.326,
                "tt-top_10_letter_intervals_correct-iqr": 0.06799999999999995,
                "tt-letter_interval_max_dev-mean": 0.40121271221261656,
                "tt-letter_interval_max_dev-std": 0.15620192604006683,
                "tt-letter_interval_max_dev-median": 0.4038602499999999,
                "tt-letter_interval_max_dev-min": 0.0,
                "tt-letter_interval_max_dev-max": 0.7237755625,
                "tt-letter_interval_max_dev-iqr": 0.11102927777777788,
                "tt-interval_until_mistake-mean": 0.34474074074074074,
                "tt-interval_until_mistake-std": 0.31707383999144867,
                "tt-interval_until_mistake-median": 0.2,
                "tt-interval_until_mistake-min": 0.081,
                "tt-interval_until_mistake-max": 1.326,
                "tt-derived_interval_until_mistake-mean": -0.19792771084337352,
                "tt-derived_interval_until_mistake-std": 0.34603844362533925,
                "tt-derived_interval_until_mistake-median": -0.039,
                "tt-derived_interval_until_mistake-min": -1.095,
                "tt-derived_interval_until_mistake-max": 0.261,
                "tt-ratio_correct_words_letters": 0.15432098765432098,
                "tt-typing_speed_slope": 0.007201260772962114,
                "tt-acc_sig_ene": 1.963212610239884,
                "tt-acc_ps_ene_x": 0.000375215931148968,
                "tt-acc_ps_ene_y": 7.449723104379295e-05,
                "tt-acc_ps_ene_z": 0.0006686787986843964,
                "tt-acc_ps_peak_x": 2.5390625,
                "tt-acc_ps_peak_y": 4.296875,
                "tt-acc_ps_peak_z": 7.03125,
                "tt-acc_ps_ent_x": 6.4890930809402,
                "tt-acc_ps_ent_y": 6.839583689879121,
                "tt-acc_ps_ent_z": 6.373085176658488,
                "tt-acc_ps_amp_x": 2.6764282845523885e-05,
                "tt-acc_ps_amp_y": 4.172290382587922e-06,
                "tt-acc_ps_amp_z": 6.429341310623917e-05,
                "tt-gyr_sig_ene": 7.656356348949542,
                "tt-gyr_ps_ene_x": 0.010577862457123851,
                "tt-gyr_ps_ene_y": 0.029376303608483187,
                "tt-gyr_ps_ene_z": 0.002118682244384485,
                "tt-gyr_ps_peak_x": 0.546875,
                "tt-gyr_ps_peak_y": 0.78125,
                "tt-gyr_ps_peak_z": 0.546875,
                "tt-gyr_ps_ent_x": 6.329138636135491,
                "tt-gyr_ps_ent_y": 5.958487680438017,
                "tt-gyr_ps_ent_z": 5.964569486227463,
                "tt-gyr_ps_amp_x": 0.001935501675479984,
                "tt-gyr_ps_amp_y": 0.003990127885503974,
                "tt-gyr_ps_amp_z": 0.00037051877221862484,
            },
        )
    ],
)
def test_process_timestamp_error(
    example_reading_processed_typing_bug_timestamp, level, expected_measures
):
    """Test: bal.processing.process_typing."""
    assert_level_values(
        example_reading_processed_typing_bug_timestamp, level, expected_measures
    )


@pytest.mark.parametrize(
    "level,expected_measures",
    [
        (
            "typing",
            {
                "tt-rt-mean": 0.876375,
                "tt-rt-std": 0.14481194702095543,
                "tt-rt_fc-mean": 0.898375,
                "tt-rt_fc-std": 0.17513768776999242,
                "tt-rt_fc_slope": 0.006477072208919195,
                "tt-rt_slope": 0.002132188030700065,
                "tt-n_correct_words": 32,
                "tt-word_duration-mean": 1.8082727272727273,
                "tt-word_duration-std": 0.4314178856346299,
                "tt-word_duration-median": 1.747,
                "tt-word_duration-min": 0.462,
                "tt-word_duration-max": 2.9250000000000003,
                "tt-word_duration-0-mean": 1.5836666666666668,
                "tt-word_duration-0-median": 1.6660000000000001,
                "tt-word_duration-0-std": 0.15844978173962046,
                "tt-word_duration-1-mean": 1.4816666666666667,
                "tt-word_duration-1-median": 1.411,
                "tt-word_duration-1-std": 0.24671103204626527,
                "tt-word_duration-2-mean": 1.869,
                "tt-word_duration-2-median": 1.6920000000000002,
                "tt-word_duration-2-std": 0.371589827632566,
                "tt-word_duration-3-mean": 1.6320000000000003,
                "tt-word_duration-3-median": 1.6520000000000001,
                "tt-word_duration-3-std": 0.04915282290977807,
                "tt-word_duration-4-mean": 1.619,
                "tt-word_duration-4-median": 1.639,
                "tt-word_duration-4-std": 0.13908270920570967,
                "tt-word_duration-5-mean": 1.8490000000000002,
                "tt-word_duration-5-median": 1.6740000000000002,
                "tt-word_duration-5-std": 0.565198195326206,
                "tt-word_duration-6-mean": 2.134666666666667,
                "tt-word_duration-6-median": 2.0620000000000003,
                "tt-word_duration-6-std": 0.2355617399607443,
                "tt-word_duration-7-mean": 1.8603333333333334,
                "tt-word_duration-7-median": 1.8410000000000002,
                "tt-word_duration-7-std": 0.07489548272982374,
                "tt-word_duration-8-mean": 2.0253333333333337,
                "tt-word_duration-8-median": 2.1420000000000003,
                "tt-word_duration-8-std": 0.3061524674624285,
                "tt-word_duration-9-mean": 2.3143333333333334,
                "tt-word_duration-9-median": 2.2070000000000003,
                "tt-word_duration-9-std": 0.5647028717240008,
                "tt-word_duration-10-mean": 2.052,
                "tt-word_duration-10-median": 2.052,
                "tt-word_duration-10-std": 0.1456639969244289,
                "tt-word_duration_slope-mean": 0.046833333333333324,
                "tt-word_duration_slope-std": 0.25476884948255873,
                "tt-word_duration_slope-median": 0.07599999999999996,
                "tt-word_duration_slope-min": -0.27433333333333354,
                "tt-word_duration_slope-max": 0.3873333333333333,
                "tt-word_duration-last_three-mean": 1.9716666666666667,
                "tt-word_duration-last_three-median": 1.949,
                "tt-word_duration-last_three-std": 0.17311653108046426,
                "tt-reaction_duration-mean": 0.05,
                "tt-reaction_duration-median": 0.05,
                "tt-reaction_duration-min": 0.05,
                "tt-reaction_duration-max": 0.05,
                "tt-words_typed_without_error": 31,
                "tt-words_typed_with_error": 1,
                "tt-consecutive_words_typed_without_error": 27,
                "tt-consecutive_words_typed_with_error": 1,
                "tt-n_incorrect_words": 1,
                "tt-ratio_letters_typed_len-mean": 1.0182291666666665,
                "tt-ratio_letters_typed_len-std": 0.07250749250618206,
                "tt-ratio_letters_typed_len-median": 1.0,
                "tt-ratio_letters_typed_len-min": 1.0,
                "tt-ratio_letters_typed_len-max": 1.3333333333333333,
                "tt-correct_letters": 199,
                "tt-letter_intervals-mean": 0.2936281407035176,
                "tt-letter_intervals-std": 0.27322949323193146,
                "tt-letter_intervals-median": 0.19,
                "tt-letter_intervals-min": 0.05,
                "tt-letter_intervals-max": 1.2870000000000001,
                "tt-letter_intervals_correct-mean": 0.2936281407035176,
                "tt-letter_intervals_correct-std": 0.27322949323193146,
                "tt-letter_intervals_correct-median": 0.19,
                "tt-letter_intervals_correct-min": 0.05,
                "tt-letter_intervals_correct-max": 1.2870000000000001,
                "tt-top_10_letter_intervals-mean": 0.9562999999999999,
                "tt-top_10_letter_intervals-std": 0.12225172477526165,
                "tt-top_10_letter_intervals-median": 0.914,
                "tt-top_10_letter_intervals-min": 0.8210000000000001,
                "tt-top_10_letter_intervals-max": 1.2870000000000001,
                "tt-top_10_letter_intervals-iqr": 0.12150000000000005,
                "tt-top_10_letter_intervals_correct-mean": 0.9562999999999999,
                "tt-top_10_letter_intervals_correct-std": 0.12225172477526165,
                "tt-top_10_letter_intervals_correct-median": 0.914,
                "tt-top_10_letter_intervals_correct-min": 0.8210000000000001,
                "tt-top_10_letter_intervals_correct-max": 1.2870000000000001,
                "tt-top_10_letter_intervals_correct-iqr": 0.12150000000000005,
                "tt-letter_interval_max_dev-mean": 0.33867123461756843,
                "tt-letter_interval_max_dev-std": 0.151504494525427,
                "tt-letter_interval_max_dev-median": 0.30248168836805556,
                "tt-letter_interval_max_dev-min": 0.0025334444444444423,
                "tt-letter_interval_max_dev-max": 0.6822760000000001,
                "tt-letter_interval_max_dev-iqr": 0.1273284829687502,
                "tt-interval_until_mistake-mean": 0.1808930817610063,
                "tt-interval_until_mistake-std": 0.07538421343609915,
                "tt-interval_until_mistake-median": 0.171,
                "tt-interval_until_mistake-min": 0.068,
                "tt-interval_until_mistake-max": 0.529,
                "tt-derived_interval_until_mistake-mean": 0.0013984375,
                "tt-derived_interval_until_mistake-std": 0.10670987525929515,
                "tt-derived_interval_until_mistake-median": -0.006500000000000001,
                "tt-derived_interval_until_mistake-min": -0.35400000000000004,
                "tt-derived_interval_until_mistake-max": 0.21000000000000002,
                "tt-ratio_correct_words_letters": 0.16,
                "tt-typing_speed_slope": 0.0021488577552766005,
            },
        )
    ],
)
def test_missing_imus(
    example_reading_processed_typing_bug_missing_imu, level, expected_measures
):
    """Test typing processing without imus data."""
    assert_level_values(
        example_reading_processed_typing_bug_missing_imu, level, expected_measures
    )


@pytest.mark.parametrize(
    "level,expected_measures",
    [
        (
            "typing",
            {
                "tt-n_autocompletion": 4,
            },
        )
    ],
)
def test_process_autocomplete(
    example_reading_processed_typing_autocomplete, level, expected_measures
):
    """Test typing processing with several autocompletions."""
    assert_level_values(
        example_reading_processed_typing_autocomplete, level, expected_measures
    )


def test_process_cancelled(example_reading_typing_cancelled):
    """Test typing processing with a typing that has been cancelled."""
    with pytest.raises(StopProcessingError):
        process_typing(example_reading_typing_cancelled)
